      Subroutine Call_Dgelsd(BMp,Vp,Cp,rcond,nobm,nob1)
***********************************************************************
*
* Subroutine to do weighted least squared fitting by call DGELSD in LAPACK
* BMp:          Modified basis set matrix 
*               (nob*nob for icop=0 and (nob-1)*(nob-1) for icop=1)
* Vp:           Modified ab initio poteintal matrix
*               (nob*1 for icop=0 and (nob-1)*1 for icop=1)
* Cp:           Optimized coefficients 
*               (nob*1 for icop=0 and (nob-1)*1 for icop=1)
* A:            Local matrix to hold BMp 
*               matrix to pass to DGELSD subroutine
* B:            Local matrix to hold Vp
*               (nob*1 for icop=0 and (nob-1)*1 for icop=1)
* NMAX:         nobm (max # of columns of A)   -VZ
* MMAX:		nobm (max # of rows of A and B)  -VZ
* nob:          # of basis function 
* nop:          # of data points
*
************************************************************************

      implicit double precision (a-h,o-z)

      INTEGER          NIN,NOUT
      INTEGER          NB,NLVL,MMAX,NMAX
      INTEGER          LDA,LDB,LIWORK,LWORK

      DOUBLE PRECISION RCOND,Ecr,esh
      INTEGER          i,info,j,k,lwrk,nob1,rank

      DOUBLE PRECISION cp(nobm),bmp(nobm,nobm),vp(nobm)
      double precision,allocatable,dimension(:,:):: A
      double precision,allocatable,dimension(:):: B, S, WORK
      integer,allocatable,dimension(:):: IWORK

*     .. External Subroutines ..
      EXTERNAL         DGELSD
*     .. Executable Statements ..

      NIN=5
      NOUT=6
      MMAX=nobm
      NMAX=nobm
      NB=64
      NLVL=10
      LDA=nobm
      LDB=nobm
      LIWORK=(3*MMAX*NLVL+11*MMAX)
      LWORK=NB*(2*MMAX+NMAX)

      allocate(A(LDA,NMAX),B(LDB),S(NMAX),WORK(LWORK))
      allocate(IWORK(LIWORK))

C IF command is kept but "nob1" can`t be larger then "nobm" -VZ
      IF (nob1.LE.MMAX .AND. nob1.LE.NMAX) THEN

*
*        Initialize matrix A
*
         do i=1,nob1
           do j=1,nob1
             A(i,j)=0.0d0
           enddo
         enddo
*
*        Form matrix A=BMP
*
         do i=1,nob1
           do j=1,nob1
             A(i,j)=BMP(i,j)
           enddo
         enddo
*
*        Initialize vector B
*
         do i=1,nob1
           B(i)=0.0d0
         enddo
*
*        Form vector B=VP
*
         do i=1,nob1
           B(i)=VP(i)
         enddo

*
*        Solve the least squares problem min( norm2(b - Ax) ) for the
*        x of minimum norm.
*
         CALL DGELSD(nob1,nob1,1,A,LDA,B,LDB,S,RCOND,RANK,WORK,LWORK,
     $               IWORK,info)
*
         IF (INFO.EQ.0) THEN
*
*           Print solution
*
C            WRITE (NOUT,*) 'Least squares solution'
C            do i=1,nob1
C              WRITE (NOUT,99996) B(i)
C            enddo
C     Pass the solution to CP
            do i=1,nob1
              CP(i)=B(i)
            enddo
*
*           Print the effective rank of A
*
C            WRITE (NOUT,*)
C            WRITE (NOUT,*) 'Tolerance used to estimate the rank of A'
C            WRITE (NOUT,99998) RCOND
C            WRITE (NOUT,*) 'Estimated rank of A'
C            WRITE (NOUT,99997) RANK
*
*           Print singular values of A
*
C            WRITE (NOUT,*)
C            WRITE (NOUT,*) 'Singular values of A'
C            WRITE (NOUT,99999) (S(I),I=1,nob)
         ELSE
            WRITE (30,*) 'The SVD algorithm failed to converge'
         END IF
      ELSE
         WRITE (NOUT,*) 'MMAX and/or NMAX too small, and/or M.GT.N'
      END IF
*
99999 FORMAT (1X,7F11.4)
99998 FORMAT (3X,1P,E11.2)
99997 FORMAT (1X,I6)
99996 FORMAT (1X,D15.8)

C close the old arrays, which no longer needed, to release memory -VZ
      deallocate(A,B,S,WORK,IWORK)

      END
