      subroutine ev2gm2(r,v2,dv2dr,pairw,imol,igrad)
***********************************************************************
*
* Subroutine to evaluate the 2-body potential energy and gradient
* for given r with two kinds of potentials:
*
* pairw=1
* generalized Morse potential with Scheme 2.
* V(r) = 0 for r -> inifity
* V(r) = De*(1-exp(-f(y)*(r-re)))^2 - De
* f(y) = c0 + c1*y + c2*y^2 + c3*y^3 + c4*y^4 + c5*y^5 + c6*y^6
* y = (r^4 - re^4)/(r^4 + re^4)
*
* imol .eq. 1   Paramemters for N2 dissociation without SEC
*               (for N4 data)
* imol .eq. 2   Paramemters for N2 dissociation with SEC
*               (for N2O2 data)
* imol .eq. 3   Paramemters for NO dissociation with SEC
*               (for N2O2 data)
* imol .eq. 4   Paramemters for O2 dissociation with SEC
*               (for N2O2 data and future O4 data)
*
***********************************************************************
* pairw=2
* generalized Morse potential with Scheme 2 for N2 in N2O2.
* V(r) = 0 for r -> inifity
* V(r) = De*(1-exp(-f(y)*(r-re)))^2 - De
* f(y) = c0 + c1*y + c2*y^2 + c3*y^3 + c4*y^4 + c5*y^5 + c6*y^6
* y = (r^4 - re^4)/(r^4 + re^4)
*
* and a polynomial based on MEG variables for NO and O2 in N2O2
* V(r) = 0 for r -> inifity
* u=exp(-(r-re)/alpha-(r-re)^2/beta))
* V(r)=-De*(c1*u^1+c2*u^2+c3*u^3+c4*u^4+c5*u^5+c6*u^6)
*
* imol .eq. 2   Paramemters for N2 dissociation with SEC
*               (for N2O2 data)
* imol .eq. 3   Paramemters for NO dissociation with SEC
*               (for N2O2 data)
* imol .eq. 4   Paramemters for O2 dissociation with SEC
*               (for N2O2 data and future O4 data)
*
***********************************************************************
* pairw=3
* 
* generalized Morse potential with Scheme 2 for N2 in N2O2.
* V_SR(r) = 0 for r -> inifity
* V_SR(r) = De*(1-exp(-f(y)*(r-re)))^2 - De
* f(y) = c0 + c1*y + c2*y^2 + c3*y^3 + c4*y^4 + c5*y^5 + c6*y^6
* y = (r^4 - re^4)/(r^4 + re^4)
*
* a polynomial based on MEG variables for NO
* V_SR(r) = 0 for r -> inifity
* u=exp(-(r-re)/alpha-(r-re)^2/beta))
* V_SR(r)=-De*(c1*u^1+c2*u^2+c3*u^3+c4*u^4+c5*u^5+c6*u^6)
*
* and for O2
* V_SR(r)= a(k)*exp(-alpha*beta^k*r^2)
* 
* imol .eq. 4   Paramemters for O2 dissociation as short-range term
*               potential for O4 data, where the long-range term
*               potential is given by D3 dispersion
*
* imol .eq. 3 Paramemters for NO dissociation as short-range term
*             potential for N2O2 data, where the long-range term
*             potential is given by D3 dispersion 
*
* imol .eq. 1 Paramemters for N2 dissociation as short-range term
*             potential for N4 data, where the long-range term
*             potential is given by D3 dispersion (used in the work
*             where MB-PIP-NN is published)
*
* imol .eq. 2  Paramemters for N2 dissociation as short-range term
*              potential for N2O2 data, where the long-range term
*              potential is given by D3 dispersion
***********************************************************************


      implicit double precision (a-h,o-z)
      integer pairw

      integer igrad,imol,k
      double precision c(0:12)
      double precision :: alpha,beta,a(0:7)
      double precision re,de,rmin,pi,rx
      double precision fy,dfdr,r,u,v2,dv2dr,y,dydr,lj,dljdr

      if (pairw .eq. 1) then
C Call generilezed Morse function to calculate the pair-wise potentials

      if (imol .eq. 1) then
C Parameter for N2 dissociation without SEC (for N4 data)
        re=1.098d0
        de=228.7d0
        c(0) = 2.70963254293d0
        c(1) = 1.32620177271d-1
        c(2) = 2.96757048793d-1
        c(3) = 1.97112432229d-1
        c(4) =-5.02002309588d-1
        c(5) = 3.80734244606d-1
        c(6) = 1.21001628750d0
      else if (imol .eq. 2) then
C Parameter for N2 dissociation with SEC (for N2O2)
        re=1.098d0
        de=228.4d0
        c(0) = 2.71405774451d0
        c(1) = 1.32757649829d-1
        c(2) = 2.66756890408d-1
        c(3) = 1.95350725241d-1
        c(4) =-4.08663480982d-1
        c(5) = 3.92451705557d-1
        c(6) = 1.13006674877d0
      else if (imol .eq. 3) then
C Parameter for NO dissociation with SEC (for N2O2)
        re=1.1508d0
C modified by ZV 3/3/14
C old  value was "d0" (149.9d0) instead of "de"
        de=152.6d0
        c(0) = 2.81134495569d0
        c(1) = 1.43241169611d-1
        c(2) = 1.35760038863d-2
        c(3) = 3.92892178507d-1
        c(4) = 9.29495534058d-1
        c(5) = 2.66966672332d-1
        c(6) =-3.68118714223d-1
      else if (imol .eq. 4) then
C Parameter for O2 dissociation with SEC (for N2O2 and O4)
        re=1.208d0
        de=120.243d0
        c(0) = 2.69132890094d0
        c(1) = 3.39550045614d-1
        c(2) = 3.46401777195d-1
        c(3) =-7.76983671636d-1
        c(4) =-3.29632972405d-1
        c(5) = 2.40883331247d0
        c(6) = 2.09264029009d0
      endif

      y=(r*r*r*r - re*re*re*re)/(r*r*r*r + re*re*re*re)

      fy = c(0) + c(1)*y + c(2)*y*y + c(3)*y*y*y + c(4)*y*y*y*y
     $   + c(5)*y*y*y*y*y + c(6)*y*y*y*y*y*y

      u=exp(-fy*(r-re))

      v2=de*(1.0d0-u)*(1.0d0-u)-de

      if (igrad .eq. 1) then
        dfdy = c(1) + 2.0d0*c(2)*y + 3.0d0*c(3)*y*y + 4.0d0*c(4)*y*y*y
     $       + 5.0d0*c(5)*y*y*y*y + 6.0d0*c(6)*y*y*y*y*y
        v = r*r*r*r + re*re*re*re
        dydr = 8.0d0*r*r*r*re*re*re*re/(v*v)
        dfdr = dfdy*dydr
        dv2dr = 2.0d0*de*(1-u)*u*(dfdr*(r-re)+fy)
      endif

      else if (pairw .eq. 2) then
      if (imol .eq. 2) then
C Parameter for N2 dissociation with SEC (for N2O2)
C This is for generalized Morse equation
        re=1.098d0
        de=228.4d0
        c(0) = 2.71405774451d0
        c(1) = 1.32757649829d-1
        c(2) = 2.66756890408d-1
        c(3) = 1.95350725241d-1
        c(4) =-4.08663480982d-1
        c(5) = 3.92451705557d-1
        c(6) = 1.13006674877d0

      else if (imol .eq. 3) then
C Parameter for NO dissociation with SEC (for N2O2)
        re=   1.1508d0
        de=   152.6d0
        c(1)=  0.322338d0
        c(2)=  5.878590d0
        c(3)= -12.790761d0
        c(4)=  13.320811d0
        c(5)= -7.516309d0
        c(6)=  1.875839d0
        c(7)= -0.052723d0
        c(8)= -0.037783d0
        c(9)=  0.48294d0   ! delta b
        c(10)= 1.98697d0   ! delta c
      else if (imol .eq. 4) then
C Parameter for O2 dissociation with SEC (for N2O2 and O4)
        re=      1.2080d0
        de=      120.243d0
        c(1)=    0.163630d0
        c(2)=   -4.695433d0
        c(3)=    56.774041d0
        c(4)=   -171.354957d0
        c(5)=    249.320090d0
        c(6)=   -195.481732d0
        c(7)=    79.575388d0
        c(8)=   -13.301027d0
        c(9)=    0.63802d0  ! delta b
        c(10)=   0.99684d0  ! delta c
      endif

      if (imol .eq. 2)  then
      ! Don wants to keep this pairwise for N2
      y=(r*r*r*r - re*re*re*re)/(r*r*r*r + re*re*re*re)

      fy = c(0) + c(1)*y + c(2)*y*y + c(3)*y*y*y + c(4)*y*y*y*y
     $   + c(5)*y*y*y*y*y + c(6)*y*y*y*y*y*y

      u=exp(-fy*(r-re))

      v2=de*(1.0d0-u)*(1.0d0-u)-de

        if (igrad .eq. 1) then
          dfdy = c(1) + 2.0d0*c(2)*y + 3.0d0*c(3)*y*y + 4.0d0*c(4)*y*y*y
     $       + 5.0d0*c(5)*y*y*y*y + 6.0d0*c(6)*y*y*y*y*y
          v = r*r*r*r + re*re*re*re
          dydr = 8.0d0*r*r*r*re*re*re*re/(v*v)
          dfdr = dfdy*dydr
          dv2dr = 2.0d0*de*(1-u)*u*(dfdr*(r-re)+fy)
        endif

      else
      !  for NO and O2
C MEG variable
       u=exp(-(r-re)/c(9)-(r-re)**2.0d0/(c(10)))
C Pair-wise potential
       v2=   - de*(c(1)*u**1.0d0 + c(2)*u**2.0d0 + c(3)*u**3.0d0 +
     $             c(4)*u**4.0d0 + c(5)*u**5.0d0 + c(6)*u**6.0d0 +
     $             c(7)*u**7.0d0 + c(8)*u**8.0d0)

C 1st derivate by r
      if (igrad .eq. 1) then
C multiplier part of the MEG derivate
       dfdr=(-2.0d0*(r-re)/c(10)-1.0d0/c(9))

       dv2dr=      - de*(c(1)*1.0d0*dfdr*u**1.0d0 +
     $                   c(2)*2.0d0*dfdr*u**2.0d0 +
     $                   c(3)*3.0d0*dfdr*u**3.0d0 +
     $                   c(4)*4.0d0*dfdr*u**4.0d0 +
     $                   c(5)*5.0d0*dfdr*u**5.0d0 +
     $                   c(6)*6.0d0*dfdr*u**6.0d0 +
     $                   c(7)*7.0d0*dfdr*u**7.0d0 +
     $                   c(8)*8.0d0*dfdr*u**8.0d0)

      endif
      endif

      else if (pairw .eq. 3) then

***********************************************************************
*
* Compute the diatomic potential of groupd-state triplet O2
*
* References: J. Chem. Phys. 132, 074307 (2010)
*
* Input:  r      interatomic distance in Angstrom
* Output: v2     potential in kcal/mol
*         dv2dr  gradient (kcal/mol)/Angstrom
*
* Parameters of analytical even-tempered Gaussian expansions for the
* ground
* state potential energy curve of O2 CBS+SR+SO+CV. Units: alpha in
* Anstromgs^-2, beta=dimensionless, a_k in milihartree.
*
***********************************************************************

      if (imol .eq. 4) then
! Original parameters
!      alpha = 0.785d0
!      beta = 1.307d0
!      a(0) = -2388.5641690d0
!      a(1) = 18086.977116d0
!      a(2) = -71760.197585d0
!      a(3) = 154738.09175d0
!      a(4) = -215074.85646d0
!      a(5) = 214799.54567d0
!      a(6) = -148395.42850d0
!      a(7) = 73310.781453d0

! Modified parameters for D3(BJ)
       alpha = 9.439784362354936d-1
       beta =  1.262242998506810d0
       a(0) = -1.488979427684798d3
       a(1) =  1.881435846488955d4
       a(2) = -1.053475425838226d5
       a(3) =  2.755135591229064d5
       a(4) = -4.277588997761775d5
       a(5) =  4.404104009614092d5
       a(6) = -2.946204062950765d5
       a(7) =  1.176861219078620d5

      v2=0.d0

      do k=0,7
       v2= v2 + a(k)*dexp(-alpha*beta**k*r**2)
      enddo
C From milihartree to kcal/mol
      v2=v2 * 627.509523475149d-3
C Compute the gradient if needed
      if (igrad.eq.1) then
       dv2dr = 0.d0
       do k=0,7
         dv2dr = dv2dr-2.d0*alpha*beta**k*r*dexp(-alpha*beta**k*r**2)
C Convert from milihartree/A to i(kcal/mol)/A
         dv2dr = dv2dr * 627.509523475149d-3
       enddo
      endif

      else if (imol .eq. 3) then
C Short range NO parameters for NO dissociation
C The long range term comes from dispersion 
C The difference between the points of the 8th order MEG NO diatomic
C potential and the dispersion corrections was refitted by a 10th
C order MEG polynomial 
        re=   1.1508d0
        de=   149.47844d0   !152.6d0
        c(1)  = -1.385343053807088d-01
        c(2)  =  1.889990874379912d+00
        c(3)  = -4.297653558650669d+00
        c(4)  =  2.143053955670876d+01
        c(5)  = -4.434782706902637d+01
        c(6)  =  4.107240828842039d+01
        c(7)  = -1.090996257595447d+01
        c(8)  = -1.068726081590712d+01
        c(9)  =  9.189735457956482d+00
        c(10) = -2.201965266358976d+00
        c(11) =  8.966018393955543d-01 ! delta b
        c(12) =  2.069542710330378d+00 ! delta c

!  for NO
C MEG variable
       u=exp(-(r-re)/c(11)-(r-re)**2.0d0/c(12))
C Pair-wise potential
       v2=0.d0

       v2= (-de)*(c(1)*u**1.0d0 + c(2)*u**2.0d0 + c(3)*u**3.0d0 +
     $            c(4)*u**4.0d0 + c(5)*u**5.0d0 + c(6)*u**6.0d0 +
     $            c(7)*u**7.0d0 + c(8)*u**8.0d0 + c(9)*u**9.0d0 +
     $            c(10)*u**10.0d0 )

C 1st derivate by r
      if (igrad .eq. 1) then
C multiplier part of the MEG derivate
       dfdr=(-2.0d0*(r-re)/c(12)-1.0d0/c(11))

       dv2dr=      - de*(c(1)*1.0d0*dfdr*u**1.0d0 +
     $                   c(2)*2.0d0*dfdr*u**2.0d0 +
     $                   c(3)*3.0d0*dfdr*u**3.0d0 +
     $                   c(4)*4.0d0*dfdr*u**4.0d0 +
     $                   c(5)*5.0d0*dfdr*u**5.0d0 +
     $                   c(6)*6.0d0*dfdr*u**6.0d0 +
     $                   c(7)*7.0d0*dfdr*u**7.0d0 +
     $                   c(8)*8.0d0*dfdr*u**8.0d0 +
     $                   c(9)*9.0d0*dfdr*u**9.0d0 +
     $                   c(10)*10.0d0*dfdr*u**10.0d0 )

      endif


      else if (imol .eq. 2)  then
      ! Don wants to keep this pairwise form for N2
      ! Parameter for N2 dissociation with SEC (for N2O2)
      ! This is for generalized Morse equation
      ! Short range N2 parameters
        re=1.098d0
        de=  224.9157d0 ! 228.4d0
        c(0) =  2.7599278840949d0  ! 2.71405774451d0
        c(1) =  0.2318898277373d0  ! 1.32757649829d-1
        c(2) =  0.1908422945648d0  ! 2.66756890408d-1
        c(3) = -0.2727504034613d0  ! 1.95350725241d-1
        c(4) = -0.5345112219335d0  !-4.08663480982d-1
        c(5) =  1.0857331617073d0  ! 3.92451705557d-1
        c(6) =  1.6339897930305d0  ! 1.13006674877d0

      y=(r*r*r*r - re*re*re*re)/(r*r*r*r + re*re*re*re)

      fy = c(0) + c(1)*y + c(2)*y*y + c(3)*y*y*y + c(4)*y*y*y*y
     $   + c(5)*y*y*y*y*y + c(6)*y*y*y*y*y*y

      u=exp(-fy*(r-re))

      v2=de*(1.0d0-u)*(1.0d0-u)-de

        if (igrad .eq. 1) then
          dfdy = c(1) + 2.0d0*c(2)*y + 3.0d0*c(3)*y*y + 4.0d0*c(4)*y*y*y
     $       + 5.0d0*c(5)*y*y*y*y + 6.0d0*c(6)*y*y*y*y*y
          v = r*r*r*r + re*re*re*re
          dydr = 8.0d0*r*r*r*re*re*re*re/(v*v)
          dfdr = dfdy*dydr
          dv2dr = 2.0d0*de*(1-u)*u*(dfdr*(r-re)+fy)
        endif

      else if (imol .eq. 1) then
C Short range N2 parameters for N2 dissociation without SEC
C (for N4 data)
C The long range term comes from dispersion
C The difference between the points of the original N2 potential
C and the dispersion corrections was refitted by the same
C polynomial as the orignal N2 was fitted

        re=1.098d0
        de=225.213d0
        c(0) =  2.7475450369759d0
        c(1) =  0.218868498415108d0
        c(2) =  0.248885765371433d0
        c(3) = -0.229295466336412d0
        c(4) = -0.653389048592838d0
        c(5) =  1.03611964035396d0
        c(6) =  1.71287482791961d0

      y=(r*r*r*r - re*re*re*re)/(r*r*r*r + re*re*re*re)

      fy = c(0) + c(1)*y + c(2)*y*y + c(3)*y*y*y + c(4)*y*y*y*y
     $   + c(5)*y*y*y*y*y + c(6)*y*y*y*y*y*y

      u=exp(-fy*(r-re))

      v2=de*(1.0d0-u)*(1.0d0-u)-de

      if (igrad .eq. 1) then
        dfdy = c(1) + 2.0d0*c(2)*y + 3.0d0*c(3)*y*y + 4.0d0*c(4)*y*y*y
     $       + 5.0d0*c(5)*y*y*y*y + 6.0d0*c(6)*y*y*y*y*y
        v = r*r*r*r + re*re*re*re
        dydr = 8.0d0*r*r*r*re*re*re*re/(v*v)
        dfdr = dfdy*dydr
        dv2dr = 2.0d0*de*(1-u)*u*(dfdr*(r-re)+fy)
      endif

      endif
      endif

      end


***************************************************************************************

      subroutine ev2gm2cas(r,v2,dv2dr,pairw,imol,igrad)
***********************************************************************
*
* Subroutine to evaluate the 2-body potential energy and gradient
* for given r:
*
* pairw=1 always
* A polinomial based on MEG variables
* V(r) = 0 for r -> inifity
* u=exp(-(r-re)/alpha-(r-re)^2/beta))
* V(r)=De*(c1*u^1+c2*u^2+c3*u^3+c4*u^4+c5*u^5+c6*u^6)
*
* imol .eq. 1   Paramemters for N2 dissociation
*               (for Yuliya's N4 data)
* imol .eq. 2   Paramemters for N2 dissociation
*               (for Zoltan's N2O2 data)
* imol .eq. 3   Paramemters for NO dissociation
*               (for Zoltan's N2O2 data)
* imol .eq. 4   Paramemters for O2 dissociation
*               (for Zoltan's N2O2 data and future O4 data)
***********************************************************************

      implicit double precision (a-h,o-z)
      integer pairw

      integer igrad,imol
      double precision c(0:10)
      double precision re,de,rmin,pi
      double precision fy,dfdr,r,u,v2,dv2dr,y,dydr,lj,dljdr

C Call MEG function to calculate the pair-wise potentials

      if (imol .eq. 1) then
C Parameter for N2 dissociation without SEC (for N4 data)
        re=1.1038d0
        de=204.07d0
         c(0) =  0.0d0
         c(1) =  0.278807015301273d0
         c(2) = -5.00836910898787d0
         c(3) =  5.80223078491159d0
         c(4) = -2.682668959071380
         c(5) =  0.573795631347316d0
         c(6) =  0.036615059234527d0
         c(7) =  0.568570812839021d0  ! delta b
         c(8) =  2.1408123632275d0   ! delta c
      else if (imol .eq. 2) then
C Parameter for N2 dissociation with SEC (for N2O2)
        re=1.1038d0
        de=204.07d0
         c(0) =  0.0d0
         c(1) =  0.278807015301273d0
         c(2) = -5.00836910898787d0
         c(3) =  5.80223078491159d0
         c(4) = -2.682668959071380
         c(5) =  0.573795631347316d0
         c(6) =  0.036615059234527d0
         c(7) =  0.568570812839021d0  ! delta b
         c(8) =  2.1408123632275d0   ! delta c
      else if (imol .eq. 3) then
C Parameter for NO dissociation with SEC (for N2O2)
        re=1.1608d0
        de=120.27d0
        c(0) =  0.0d0
        c(1) =  0.194275390167708d0
        c(2) = -4.36015458728043d0
        c(3) =  4.4099547554917d0
        c(4) = -1.20962396618733d0
        c(5) = -0.351529608164654d0
        c(6) =  0.318032772693513d0
        c(7) =  0.56073689765999d0  ! delta b
        c(8) =  1.31977539253513d0  ! delta c
      else if (imol .eq. 4) then
C Parameter for O2 dissociation with SEC (for N2O2 and O4)
        re=1.2154d0
        de=89.46d0
        c(0) =  0.0d0
        c(1) =  0.790813920036405d0
        c(2) = -5.99599976032215d0
        c(3) =  6.01015545554381d0
        c(4) = -2.21630024345457d0
        c(5) =  0.364871035471d0
        c(6) =  0.0384601625499182d0
        c(7) =  0.552313132815711d0   ! delta b
        c(8) =  2.60590408671283d0  ! delta c
      endif

C MEG variable
       u=exp(-(r-re)/c(7)-(r-re)**2.0d0/(c(8)))
C Pair-wise potential
       v2=  de*(c(1)*u**1.0d0 + c(2)*u**2.0d0 + c(3)*u**3.0d0 +
     $          c(4)*u**4.0d0 + c(5)*u**5.0d0 + c(6)*u**6.0d0)

C 1st derivate by r
      if (igrad .eq. 1) then
C multiplier part of the MEG derivate
       dfdr=(-2.0d0*(r-re)/c(8)-1.0d0/c(7))

       dv2dr=   de*(c(1)*1.0d0*dfdr*u**1.0d0 +
     $              c(2)*2.0d0*dfdr*u**2.0d0 +
     $              c(3)*3.0d0*dfdr*u**3.0d0 +
     $              c(4)*4.0d0*dfdr*u**4.0d0 +
     $              c(5)*5.0d0*dfdr*u**5.0d0 +
     $              c(6)*6.0d0*dfdr*u**6.0d0 )

      endif

      end


 
