      subroutine coordconvt(Coord,Dist,X)      
***********************************************************************
*  The subroutine reads 
*  ICoord = 0   6 variables
*  ICoord = 1   12 Cartesian coordinates
*  Calculates the inter-atomic distances by using XYZ coordinates 
***********************************************************************
      implicit double precision (a-h,o-z)

      common /inout/ icoord,ifmt,iout,isan

      integer i
      double precision X(12),Coord(12),Dist(6)
      double precision r1,r2,d,theta1,theta2,phi,pi,zero,half
      data zero/0.0d0/,half/0.5d0/,pi/3.14159265358979d0/

C
C Use Jacob coordinates to calculate XYZ coordinates for ICoord = 0
C
      if (icoord .eq. 0) then
        r1=Coord(1)
        r2=Coord(2)
        d=Coord(3)
        theta1=Coord(4)*pi/180.0d0
        theta2=Coord(5)*pi/180.0d0
        phi=Coord(6)*pi/180.0d0

***********************************************************************
*  The coordinate of A4
*  A1---A2      
*  .    .      
*  .    .     
*  A3---A4
*  A1   X1,X2,X3
*  A2   X4,X5,X6
*  A3   X7,X8,X9
*  A4   X10,X11,X12
***********************************************************************

        X(1)=half*r1*Sin(theta1)
        X(2)=zero
        X(3)=half*d - half*r1*Cos(theta1)
        X(4)=-half*r1*Sin(theta1)
        X(5)=zero
        X(6)=half*d + half*r1*Cos(theta1)
        X(7)=half*r2*Sin(theta2)*Cos(phi)
        X(8)=half*r2*Sin(theta2)*Sin(phi)
        X(9)=-half*d + half*r2*Cos(theta2)
        X(10)=-half*r2*Sin(theta2)*Cos(phi)
        X(11)=-half*r2*Sin(theta2)*Sin(phi)
        X(12)=-half*d - half*r2*Cos(theta2)
C
C Pass Cartesian cordinates Coord(12) to X(12) for ICoord = 1
      else if (icoord .eq. 1) then
        do i=1,12
          X(i)=Coord(i)
        enddo
      endif

***********************************************************************
*  Now, calculate the inter-atomic distance
*  r1 = r(A1A2)    r2 = r(A1A3)
*  r3 = r(A1A4)    r4 = r(A2A3)
*  r5 = r(A2A4)    r6 = r(A3A4)       
***********************************************************************

      Dist(1)=Sqrt((X(4)-X(1))*(X(4)-X(1)) + (X(5)-X(2))*(X(5)-X(2))
     $           + (X(6)-X(3))*(X(6)-X(3)))
      Dist(2)=Sqrt((X(7)-X(1))*(X(7)-X(1)) + (X(8)-X(2))*(X(8)-X(2)) 
     $           + (X(9)-X(3))*(X(9)-X(3)))
      Dist(3)=Sqrt((X(10)-X(1))*(X(10)-X(1)) + (X(11)-X(2))*(X(11)-X(2))
     $           + (X(12)-X(3))*(X(12)-X(3)))
      Dist(4)=Sqrt((X(7)-X(4))*(X(7)-X(4)) + (X(8)-X(5))*(X(8)-X(5))
     $           + (X(9)-X(6))*(X(9)-X(6)))
      Dist(5)=Sqrt((X(10)-X(4))*(X(10)-X(4)) + (X(11)-X(5))*(X(11)-X(5)) 
     $           + (X(12)-X(6))*(X(12)-X(6)))
      Dist(6)=Sqrt((X(10)-X(7))*(X(10)-X(7)) + (X(11)-X(8))*(X(11)-X(8)) 
     $           + (X(12)-X(9))*(X(12)-X(9)))

      return

      end
