      subroutine frmbmp(bm,bmp,vabit,vp,v2b,nop,nobm,nob,nob1,d)
***********************************************************************
* Subroutine to form the modified basis function matrix BMp(nob1,nob1)
* Normal Equation: BMp(nob1,nob1)*Cp(nob1,1)=Vp(nob1,1)
* a(6):		Nonlinear parameters in Morse terms(Angstrom)
* re(6):	Equilibrium bond length(Angstrom)
* nop:		number of points
* nom:  	number of monomials
* nob:  	number of basis functions(polynomials)
* nob1:         effective number of basis functions
***********************************************************************
      
      implicit double precision (a-h,o-z)

      common /optn/ iatyp,icop,imethd,iwop,ityp,nfun,imga,imgb,imgc,imgd
      common /energy/ ecr,esh,epow,elimit,erange(4),grange(4),v0

      integer i,j,k,nob1
      double precision bm(nop,nobm),bmp(nobm,nobm)
      double precision v2b(nop),vabit(nop),vp(nobm)
C      double precision v1(nop),w(nop)
      double precision,allocatable,dimension(:):: v1,w
      double precision,allocatable,dimension(:,:):: bm3

      double precision,dimension(:):: d(nop)

      allocate(v1(nop),w(nop),bm3(nop,nobm))

      do i=1,nop
       do j=1,nobm
        bm3(i,j)=0.0d0
       enddo
       v1(i)=0.0d0
       w(i)=0.0d0
      enddo
      do i=1,nobm
        do j=1,nobm
          bmp(i,j)=0.0d0
        enddo
      enddo
C
C Substract V2B from Vabit to form V1 when imethd = 2
C
      if (imethd .eq. 2) then
        nob1=nob
        do i=1,nop
          v1(i)=vabit(i)-v2b(i)
          do j=1,nob1
            bm3(i,j)=bm(i,j)
          enddo
        enddo
C ICop will be disabled when IMethd = 2, GO TO statement to skip ICop
C section.
C        write(*,*) "The generalized Morse potential with Scheme 2 is"
C        write(*,*) "used for the 2-body interactions."
      else if ((imethd .eq. 0) .or. (imethd .eq. 1)) then  
C
C Form intermediate basis matrix bm3 based on ICop
C whether constraint is used or not for IMethd =0 or 1
C
C Without using constraints
        if (icop .eq. 0) then
          nob1=nob
          do i=1,nop
            v1(i)=vabit(i)
            do j=1,nob1
              bm3(i,j)=bm(i,j)
            enddo
          enddo
C Constraint C0=V0
        else if(icop .eq. 1) then
          do i=1,nop
            v1(i)=vabit(i)-v0
            nob1=nob-1
            do j=1,nob1
              bm3(i,j)=bm(i,j+1)
            enddo
          enddo
C Print error information
        else
          write(*,*) "Only 0 or 1 is allowed for icop!"  
          stop
        endif
      else if (imethd .eq. 3) then
        nob1=nob
        do i=1,nop
          v1(i)=vabit(i)
          do j=1,nob1
            bm3(i,j)=bm(i,j)
          enddo
        enddo
      endif

C without using weighting factor
      if (iwop .eq. 0) then
        do i=1,nop
          w(i)=1.0d0
        enddo 
C For iwop=1, use w(i)=1.0 for Vabit(i) < Ecr
C             use w(i)=(Ecr/Vabit(i))^2
      else if (iwop .eq. 1) then
        do i=1,nop
          if ((vabit(i)-esh) .le. ecr) then
            w(i)=1.0d0
          else
C mod ecr for O3 system
            w(i)=ecr**epow/((vabit(i)-esh)**epow)
          endif
        enddo
      else if (iwop .eq. 2) then
        do i=1,nop
          if ((vabit(i)-esh) .le. ecr) then
            w(i)=1.0d0*d(i)
          else
C mod ecr for O3 system
            w(i)=d(i)*(ecr**epow)/((vabit(i)-esh)**epow)
          endif
        enddo
C For iwop=3, use w(i)=1.0 for Vabit(i) < Ecr
C             use w(i)=(Vabit(i)/Ecr)^2
      else if (iwop .eq. 3) then
        do i=1,nop
          if (abs(vabit(i)) .le. ecr) then
            w(i)=1.0d0
          else
            w(i)=(vabit(i)**epow)/(ecr**epow)
          endif
        enddo
      else if (iwop .eq. 4) then
        do i=1,nop
          if (abs(vabit(i)) .le. ecr) then
            w(i)=1.0d0*d(i)
          else
            w(i)=d(i)*(vabit(i)**epow)/(ecr**epow)
          endif
        enddo
      else
        write(*,*) "Only 0, 1, 2, 3, and 4 is allowed for iwop!" 
      endif

C     
C Obtained normal equation 
C

C Form BM'
       do i=1,nob1
         do j=1,nob1
           do k=1,nop
             bmp(i,j)=bmp(i,j) + bm3(k,i)*w(k)*bm3(k,j)
           enddo
         enddo
       enddo

C Form V1'
       do i=1,nob1
         do k=1,nop
           vp(i)=vp(i) + bm3(k,i)*w(k)*v1(k)
         enddo
       enddo

C close the old arrays, which no longer needed, to release memory -VZ
      deallocate(v1,w,bm3)

      end 
