      program PESfit
***********************************************************************
* Program to do linear least-square fitting using permutation
* invariant Morse/Gauss-type basis functions
* MaxNOS:       Maximum allowed number of electronic states
* arng:	Nonlinear parameters in Morse/Gauss terms (6)
* brng:       Nonlinear parameters in Morse/Gauss terms (6) for
*               function B  -VZ
* re:   	Equilibrium bond lengths (6)
* R:		Interatomic bond distance matrix (nop*6)
* Vabit:	Ab initio energy array (nop*1)
* Vcas:         CASSCF energy array (nop*1)
* Vcor:         Correlation energy array (nop*1)
* Vfit:		Fitted energy array (nop*1)
* nop:		Number of points used in the fitting
* nom:		Number of monomials in numb.f  -VZ
* nob:		Number of basis functions(polynomials) from first numb.f
*               but decread by frmbmx2b.f or frmbmxnc.f -VZ  043014
*               or frmbmx3b.f -YS 062118
* BM:		Basis functions matrix (nop*nob)  -VZ
* C:		Matrix containing the coefficients (nob*1) -VZ
* Cab:          Absoult values of coefficients (nob*1) -VZ 043014
* V2b:          Sum of diatomic energies -VZ
* V2bAB,V2bAC, \ 
* V2bAD,V2bBC,  : Individual diatomic energies -VZ
* V2bBD,V2bCD  /
* pairw:        Select the way of the calculation of pairwise PECs
*
***********************************************************************
      
      implicit double precision (a-h,o-z) 

      common /inout/ icoord,ifmt,iout,isan
      common /optn/ iatyp,icop,imethd,iwop,ityp,nfun,imga,imgb,imgc,imgd
      common /energy/ ecr,esh,epow,elimit,erange(4),grange(4),v0
      common /msprmt/ arng(6),brng(6),crng(6),drng(6)
      common /msprmt3/ erng(6),frng(6),grng(6),hrng(6)
      common /msprmt2/ ra(6),rb(6),rc(6),rd(6),re(6),rf(6),rg(6),rh(6)
      common /errlist/ smiel, smael, sael
      common /cor/ pairw,ecor,ired,nred

      character (len=50) uinp,abnu,outu,erru,cffu,vfu,vfeu,vfe2u,
     $                   abnucas,abnucor,vfcas,vfcor,cffucas,cffucor
      common /filnam/ uinp,abnu,outu,erru,cffu,vfu,vfeu,vfe2u,
     $                   abnucas,abnucor,vfcas,vfcor,cffucas,cffucor

      integer i,j,nobh,stat,iout
      integer iordera,iorderb,iorderc,iorderd
      integer nob,nom,nop,nopc
      integer noma,nomb,nomc,nomd
      integer noba,nobb,nobc,nobd
      integer nobma,nobmb,nobmc,nobmd
      integer esw
      integer pairw,ecor,ired,nred

      character (len=50) kk

      double precision rcond
      double precision dist(6),distc(6)
      double precision dvecm,dcvecm
      double precision V2ABx,V2ACx,V2ADx,V2BCx,
     $ V2BDx,V2CDx
      double precision V2corABx,V2corACx,V2corADx,V2corBCx,
     $ V2corBDx,V2corCDx

      double precision,allocatable,dimension(:)::bma,bmb,bmc,bmd
      double precision,allocatable,dimension(:,:):: r,X,Jacob,
     $                                              rcr,Xc,Jacobc
      double precision,allocatable,dimension(:,:)::BM,
     $                                             BMcas,BMcor
      double precision,allocatable,dimension(:):: Vabit,Vcas,Vcor,Vx,
     $   Vfit,Vcasf,Vcorf,
     $   V2b,V2bcas,V2bcor,
     $   V2bAB,V2bAC,V2bAD,V2bBC,V2bBD,V2bCD,
     $   V2bcorAB,V2bcorAC,V2bcorAD,V2bcorBC,V2bcorBD,V2bcorCD,
     $   C,Cab,Ccor,Ccab,
     $   Vcasfx,Vcorfx,
     $   dvec,dcvec
      double precision,allocatable,dimension(:,:):: rsrt,rcsrt
     
      character (len=40) filen
      integer irec,ierr
      logical fe1,fe2,fe3,fe4,fe5 
      character (len=70) text 

      integer nops(6)
      double precision rmsd(6),sd(6)
      double precision mud(6),ud(6)

      double precision disp
      double precision,allocatable,dimension(:):: disper

      write(*,'(a,$)') " Write the name of input file (w/o extension):"
      read(*,*) filen 
!     call get_command_argument(1,filen)
!     if (LEN_TRIM(filen) == 0) then
!        write(*,*) "input file extension is missing, exit"
!        stop
!     else
!        write(*,*) "input file extension: ",filen
!     endif

      write(uinp,fmt='(a,a)')  trim(adjustl(filen)),".inp"    
      write(abnu,fmt='(a,a)')  trim(adjustl(filen)),".abn" 
      write(outu,fmt='(a,a)')  trim(adjustl(filen)),".out" 
      write(erru,fmt='(a,a)')  trim(adjustl(filen)),".err" 
      write(cffu,fmt='(a,a)')  trim(adjustl(filen)),".cff" 
      write(vfu,fmt='(a,a)')   trim(adjustl(filen)),".vf0" 
      write(vfeu,fmt='(a,a)')  trim(adjustl(filen)),".vf1" 
      write(vfe2u,fmt='(a,a)') trim(adjustl(filen)),".vf2" 
      write(abnucas,fmt='(a,a)')  trim(adjustl(filen)),".abnll"
      write(abnucor,fmt='(a,a)')  trim(adjustl(filen)),".abnhl"
      write(vfcas,fmt='(a,a)')  trim(adjustl(filen)),".vfll"
      write(vfcor,fmt='(a,a)')  trim(adjustl(filen)),".vfhl"
      write(cffucas,fmt='(a,a)')  trim(adjustl(filen)),".cffll"
      write(cffucor,fmt='(a,a)')  trim(adjustl(filen)),".cffhl"

      open(unit=30,file=trim(adjustl(outu)))

      write(30,*) ''
      write(30,*) "Permuationally invariant polynomial fitting 
     &program"
      write(30,*) ''
      write(30,*) "PIPList 2022"
      write(30,*) "written by Ke R. Yang, Zoltan Varga,
     $ Kelsey A. Parker,"
      write(30,*) "Yinan Shu, and Donald G. Truhlar."
      write(30,*) ''
C Print date and time
      call getdat(iyr,imon,iday)             
      call gettim(ihr,imin,isec,ihund)            
      write(30,1000) iyr,imon,iday,ihr,imin,isec 
 1000 Format(' Enter PIPFit at ',I4, '/',I2,'/',I2,','
     $       ,I2,':',I2,':',I2,'.')
      write(30,*) ''
      write(30,*) ''

C read the input file
      call readinp(iordera,iorderb,iorderc,iorderd,rcond,
     $             pairw,ecor,ired,nred)

C Form allocatable arrays for input data with 'nop' lines -VZ

       if (ecor .le. 1) then
       inquire(file=trim(adjustl(abnu)),EXIST=fe1)
       inquire(file="main.abn",EXIST=fe2)
          if (fe1 .EQV. .True.) then
           open (unit=20, file=abnu)
           write(30,*) trim(adjustl(abnu))," will be read."
          else
            if (fe2 .EQV. .True.) then
             open (unit=20, file="main.abn")
             write(30,*) trim(adjustl(abnu))," does not exist, but"
             write(30,*) " main.abn will be read."
            else
             write(*,*) " Data file does not exist!"
             stop
            endif
          endif
       else if (ecor .ge. 2) then
       inquire(file=trim(adjustl(abnucas)),EXIST=fe1)
       inquire(file="main.abnll",EXIST=fe2)
       inquire(file=trim(adjustl(abnucor)),EXIST=fe3)
       inquire(file="main.abnhl",EXIST=fe4)    
          if (ecor .eq. 4) then    
              if ((fe1 .NEQV. .True.).and.(fe2 .NEQV. .True.).and.
     $            (fe3 .NEQV. .True.).and.(fe4 .NEQV. .True.)) then
                write(*,*) " Data files do not exist!"
                stop
              endif
             if (fe1 .EQV. .True.) then
              open (unit=20, file=abnucas)
              write(30,*) trim(adjustl(abnucas))," will be read."
             else
               if (fe2 .EQV. .True.) then
                open (unit=20, file="main.abncas")
                 write(30,*) trim(adjustl(abnucas)),
     $           " does not exist, but"
                 write(30,*) " main.abnll will be read."
               else
                write(*,*) " Data file for Ell does not exist!"
                stop
               endif
             endif
             if (fe3 .EQV. .True.) then
              open (unit=25, file=abnucor)
              write(30,*) trim(adjustl(abnucor))," will be read."
             else
               if (fe4 .EQV. .True.) then
                open (unit=25, file="main.abnhl")
                 write(30,*) trim(adjustl(abnucor)),
     $            " does not exist, but"
                 write(30,*) " main.abnhl will be read."
               else
                write(*,*) " Data file for Ehl does not exist!"
               stop
               endif
             endif
          else if (ecor .eq. 2) then
             if (fe1 .EQV. .True.) then
              open (unit=20, file=abnucas)
              write(30,*) trim(adjustl(abnucas))," will be read."
             else
               if (fe2 .EQV. .True.) then
                open (unit=20, file="main.abnll")
                 write(30,*) trim(adjustl(abnucas)),
     $           " does not exist, but"
                 write(30,*) " main.abnll will be read."
               else
                write(*,*) " Data file for Ell does not exist!"
                stop
               endif
             endif
          else if (ecor .eq. 3) then
             if (fe3 .EQV. .True.) then
              open (unit=25, file=abnucor)
              write(30,*) trim(adjustl(abnucor))," will be read."
             else
               if (fe4 .EQV. .True.) then
                open (unit=25, file="main.abnhl")
                 write(30,*) trim(adjustl(abnucor)),
     $            " does not exist, but"
                 write(30,*) " main.abnhl will be read."
               else
                write(*,*) " Data file for Ehl does not exist!"
               stop
               endif
             endif
          endif
       endif

       if (ecor .le. 2) then
         nop=0
         do i=1,20
          read(20,*)
         end do
         do
          read(20,*,iostat=stat)
          if (.not. (stat==0)) then
           exit
          end if
          nop= nop+1
         end do
         close(20)
       else if (ecor .eq. 3) then
         nop=0
         nopc=0
         do i=1,20
          read(25,*)
         end do
         do
          read(25,*,iostat=stat)
          if (.not. (stat==0)) then
           exit
          end if
          nopc= nopc+1
         end do
         close(25)
       else if (ecor .eq. 4) then
         nop=0
         nopc=0
         do i=1,20
          read(20,*)
          read(25,*)
         end do
         do
          read(20,*,iostat=stat)
          if (.not. (stat==0)) then
           exit
          end if
          nop= nop+1
         end do
         do
          read(25,*,iostat=stat)
          if (.not. (stat==0)) then
           exit
          end if
          nopc= nopc+1
         end do
         close(20)
         close(25)
       endif

C divide the size of the lines by 5 if Cartasian coordinates were read
C (5 lines per gemetry)
      if (icoord .eq. 1) then
       nop= nop/5
       if (ecor .ge. 3) then
        nopc= nopc/5
       endif
      else
       continue
      endif

      if (ecor .eq. 4) then
        write(30,*) trim(adjustl(vfcas)),' and ', trim(adjustl(vfcor)),
     $ ' files will be generated.'
      else if (ecor .eq. 3) then
        write(30,*) trim(adjustl(vfcor)),
     $ ' file will be generated.'
      else if (ecor .eq. 2) then
        write(30,*) trim(adjustl(vfcas)),
     $ ' file will be generated.'
      else if (ecor .le. 1) then
        if (iout .eq. 0) then
          write(30,*) trim(adjustl(vfu)),' file will
     $    be generated.'
         else if (iout .eq. 1) then
          write(30,*) trim(adjustl(vfu)),' and ',
     $    trim(adjustl(vfeu)),' files will be generated.'
         else
          write(30,*) trim(adjustl(vfu)),', ',
     $   trim(adjustl(vfeu)),', and',trim(adjustl(vfe2u)),
     $   ' files will be generated.'
         endif
      endif

      if (isan .eq. 1) then
        write(30,*)
        write(30,'(a)') ' Statistical analysis of coeffs will be made.'
       else
        continue
      endif

      if (ifmt .eq. 0) then
        write(30,*)
        write(30,'(a)') ' Standard format of input file.'
      else 
        write(30,*)
        write(30,'(a)') ' Input with index of each data points.'
      endif     

      write(30,*)
      write(30,2005) elimit,ecr,epow,v0
      write(30,2006) iwop
      write(30,2007) icop
      write(30,2008) icoord
      write(30,*) 

 2005 Format(' Elimit = ',F8.2,' kcal/mol,',' Ecritical = ',F6.2,
     $' Epower =',F4.1,' kcal/mol, V0 =',F6.2,' kcal/mol')
 2006 Format(' Weight option IWop = ',I2)
 2007 Format(' Constraint option ICop = ',I2)
 2008 Format(' Coordinate option ICoord = ',I2)

      if (ecor .eq. 0) then
         write(30,*) " Vtotal will be fitted."
       if (nfun .eq. 1) then
         write(30,2010)
        if (imga .eq. 1) then
         write(30,2011)
        else if (imga .eq. 2) then
         write(30,2012)        
        else if (imga .eq. 3) then
         write(30,2013)        
        else if (imga .eq. 4) then
          write(30,2014)       
        else if (imga .eq. 5) then
          write(30,2015)
        endif
       else if (nfun .eq. 2) then 
         write(30,2018)   
        if (imga .eq. 1) then
         write(30,2011)
        else if (imga .eq. 2) then
         write(30,2012)        
        else if (imga .eq. 3) then
         write(30,2013)        
        else if (imga .eq. 4) then
         write(30,2014)
        else if (imga .eq. 5) then
         write(30,2015)
        endif
         write(30,*) " and"
        if (imgb .eq. 1) then
         write(30,2011)
        else if (imgb .eq. 2) then
         write(30,2012)        
        else if (imgb .eq. 3) then
         write(30,2013)        
        else if (imgb .eq. 4) then
         write(30,2014)
        else if (imgb .eq. 5) then
          write(30,2015)
        endif
       endif
      else if (ecor .eq. 1) then
         write(30,*) " Vll + Vhl will be fitted."
         write(30,*) " Vll:"
       if (nfun .eq. 1) then
         write(30,2010)
        if (imga .eq. 1) then
         write(30,2011)
        else if (imga .eq. 2) then
         write(30,2012)        
        else if (imga .eq. 3) then
         write(30,2013)        
        else if (imga .eq. 4) then
          write(30,2014)
        else if (imga .eq. 5) then
          write(30,2015)
        endif
       else if (nfun .eq. 2) then 
         write(30,2018)   
        if (imga .eq. 1) then
         write(30,2011)
        else if (imga .eq. 2) then
         write(30,2012)        
        else if (imga .eq. 3) then
         write(30,2013)        
        else if (imga .eq. 4) then
         write(30,2014)
        else if (imgb .eq. 5) then
          write(30,2015)
        endif
         write(30,*) " and"
        if (imgb .eq. 1) then
         write(30,2011)
        else if (imgb .eq. 2) then
         write(30,2012)        
        else if (imgb .eq. 3) then
         write(30,2013)        
        else if (imgb .eq. 4) then
         write(30,2014)
        else if (imgb .eq. 5) then
          write(30,2015)
        endif
       endif
         write(30,*) " Vhl:"
        if (imgc .eq. 1) then
         write(30,2011)
        else if (imgc .eq. 2) then
         write(30,2012)        
        else if (imgc .eq. 3) then
         write(30,2013)        
        else if (imgc .eq. 4) then
         write(30,2014)
        else if (imgc .eq. 5) then
          write(30,2015)
        endif
      else if (ecor .eq. 2) then
         write(30,*) " Vll will be fitted."
       if (nfun .eq. 1) then
         write(30,2010)
        if (imga .eq. 1) then
         write(30,2011)
        else if (imga .eq. 2) then
         write(30,2012)        
        else if (imga .eq. 3) then
         write(30,2013)        
        else if (imga .eq. 4) then
          write(30,2014) 
        else if (imga .eq. 5) then
          write(30,2015)
        endif
       else if (nfun .eq. 2) then 
         write(30,2018)   
        if (imga .eq. 1) then
         write(30,2011)
        else if (imga .eq. 2) then
         write(30,2012)        
        else if (imga .eq. 3) then
         write(30,2013)        
        else if (imga .eq. 4) then
         write(30,2014)
        else if (imga .eq. 5) then
          write(30,2015)
        endif
         write(30,*) " and"
        if (imgb .eq. 1) then
         write(30,2011)
        else if (imgb .eq. 2) then
         write(30,2012)        
        else if (imgb .eq. 3) then
         write(30,2013)        
        else if (imgb .eq. 4) then
         write(30,2014)
        else if (imgb .eq. 5) then
          write(30,2015)
        endif
       endif
      else if (ecor .eq. 3) then
         write(30,*) " Vhl will be fitted."
        if (imgc .eq. 1) then
         write(30,2011)
        else if (imgc .eq. 2) then
         write(30,2012)        
        else if (imgc .eq. 3) then
         write(30,2013)        
        else if (imgc .eq. 4) then
         write(30,2014)
        else if (imgc .eq. 5) then
          write(30,2015)
        endif
      else if (ecor .eq. 4) then
         write(30,*) " Vll, Vhl will be fitted."
         write(30,*) " Vll:"
       if (nfun .eq. 1) then
         write(30,2010)
        if (imga .eq. 1) then
         write(30,2011)
        else if (imga .eq. 2) then
         write(30,2012)        
        else if (imga .eq. 3) then
         write(30,2013)        
        else if (imga .eq. 4) then
          write(30,2014) 
        else if (imga .eq. 5) then
          write(30,2015)
        endif
       else if (nfun .eq. 2) then 
         write(30,2018)   
        if (imga .eq. 1) then
         write(30,2011)
        else if (imga .eq. 2) then
         write(30,2012)        
        else if (imga .eq. 3) then
         write(30,2013)        
        else if (imga .eq. 4) then
         write(30,2014)
        else if (imga .eq. 5) then
          write(30,2015)
        endif
         write(30,*) " and"
        if (imgb .eq. 1) then
         write(30,2011)
        else if (imgb .eq. 2) then
         write(30,2012)        
        else if (imgb .eq. 3) then
         write(30,2013)        
        else if (imgb .eq. 4) then
         write(30,2014)
        else if (imgb .eq. 5) then
          write(30,2015)
        endif
       endif
         write(30,*) " Vhl:"
        if (imgc .eq. 1) then
         write(30,2011)
        else if (imgc .eq. 2) then
         write(30,2012)        
        else if (imgc .eq. 3) then
         write(30,2013)        
        else if (imgc .eq. 4) then
         write(30,2014)
        else if (imgc .eq. 5) then
          write(30,2015) 
        endif
       endif

 2010 Format(" NFun= 1")
 2011 Format(" - Morse function will be used.")
 2012 Format(" - Gauss function will be used.")
 2013 Format(" - MEG function will be used.")
 2014 Format(" - Sech function will be used.")
 2015 Format(" - Sin function will be used.")
 2018 Format(" NFun= 2")

 2019 Format(" RCond = ",D10.3," used in dgelsd subrounte.")

       if (ecor .eq. 0) then
         if (nfun .eq. 1) then
           write(30,fmt='(a,I2)') "   Iorder  =  ",iordera
         else
           write(30,fmt='(a,I2)') "   IorderA =  ",iordera
           write(30,fmt='(a,I2)') "   IorderB =  ",iorderb
         endif  
       else if (ecor .eq. 1) then
         if (nfun .eq. 1) then
           write(30,fmt='(a,I2)') "   IorderA =  ",iordera
         else
           write(30,fmt='(a,I2)') "   IorderA =  ",iordera
           write(30,fmt='(a,I2)') "   IorderB =  ",iorderb
         endif
           write(30,fmt='(a,I2)') "   IorderC =  ",iorderc
       else if (ecor .eq. 2) then
         if (nfun .eq. 1) then
           write(30,fmt='(a,I2)') "   IorderA =  ",iordera
         else
           write(30,fmt='(a,I2)') "   IorderA =  ",iordera
           write(30,fmt='(a,I2)') "   IorderB =  ",iorderb
         endif 
       else if (ecor .eq. 3) then
           write(30,fmt='(a,I2)') "   IorderC =  ",iorderc  
       else if (ecor .eq. 4) then
         if (nfun .eq. 1) then
           write(30,fmt='(a,I2)') "   IorderA =  ",iordera
         else
           write(30,fmt='(a,I2)') "   IorderA =  ",iordera
           write(30,fmt='(a,I2)') "   IorderB =  ",iorderb
         endif
           write(30,fmt='(a,I2)') "   IorderC =  ",iorderc   
       endif

      write(30,*)
      write(30,2019) rcond

      if (imethd .eq. 0) then
        write(30,2020)
 2020 Format(" IMethd = 0, the Bowman-Braams approach will be used.")
      else if (imethd .eq. 1) then
        write(30,2021)
 2021 Format(" IMethd = 1, the unconnected terms will be removed from
     $  Bowman-Braams' expansions.")
      else if (imethd .eq. 2) then
        if (pairw .eq. 1) then 
          write(30,2022)
 2022 Format(" IMethd = 2, the generalized Morse-potentials will be
     $ used for the pairwise interactions.")
        else if (pairw .eq. 2) then
          write(30,2023)
 2023 Format(" IMethd = 2, the MEG potentials will be
     $ used for the pairwise interactions.")
        else
          write(30,2024)
 2024 Format(" IMethd = 2, the SR+LR potentials will be
     $ used for the pairwise interactions.")
        endif
      else if (imethd .eq. 3) then
        write(30,2025)
 2025 Format(" IMethd = 3, the 3-body unconnected terms will be
     $ removed from Bowman-Braams' expansions.")
      endif
      write(30,*) "Succeed to read parameters from file ",
     $ trim(adjustl(uinp))," ..."
      write(30,*)

      if (ecor .le. 2) then
      write(30,2040), nop      
      else if (ecor .eq. 3) then
      write(30,2041), nopc
      else if (ecor .eq. 4) then
      write(30,2040), nop
      write(30,2042), nopc
      endif

 2040 Format(' Number of Ell data points: ',I7)
 2041 Format(' Number of Ehl data points: ',I7)
 2042 Format(' Number of data points: ',I7)
      write(30,*)

      noma=0
      nomb=0
      nomc=0
      nomd=0

      nobma=0
      nobmb=0
      nobmc=0
      nobmd=0

      noba=0
      nobb=0
      nobc=0
      nobd=0

C allocation borders for two different order functions

       if (ecor .eq. 0) then
        if (nfun .eq. 1) then
          call numb(noma,nobma,iordera)
          nom=noma
          nobm=nobma
        else if (nfun .eq. 2) then
          call numb(noma,nobma,iordera)
          call numb(nomb,nobmb,iorderb)
          nom=noma+nomb
          nobm=nobma+nobmb
        endif 
       else if (ecor .eq. 1) then
        if (nfun .eq. 1) then
          call numb(noma,nobma,iordera)
          nom=noma
          nobm=nobma
        else if (nfun .eq. 2) then
          call numb(noma,nobma,iordera)
          call numb(nomb,nobmb,iorderb)
          nom=noma+nomb
          nobm=nobma+nobmb
        endif
          call numb(nomc,nobmc,iorderc)
       else if (ecor .eq. 2) then
        if (nfun .eq. 1) then
          call numb(noma,nobma,iordera)
          nom=noma
          nobm=nobma
        else if (nfun .eq. 2) then
          call numb(noma,nobma,iordera)
          call numb(nomb,nobmb,iorderb)
          nom=noma+nomb
          nobm=nobma+nobmb
        endif 
       else if (ecor .eq. 3) then
          call numb(nomc,nobmc,iorderc)
       else if (ecor .eq. 4) then
        if (nfun .eq. 1) then
          call numb(noma,nobma,iordera)
          nom=noma
          nobm=nobma
        else if (nfun .eq. 2) then
          call numb(noma,nobma,iordera)
          call numb(nomb,nobmb,iorderb)
          nom=noma+nomb
          nobm=nobma+nobmb
        endif 
          call numb(nomc,nobmc,iorderc)
       endif 
C      write(*,*) "noma = ",noma," nobma = ",nobma
C      write(*,*) "nomb = ",nomb," nobmb = ",nobmb
C      write(*,*) "nom  = ",nom ," nobm  = ",nobm 
C      write(*,*) "nomc = ",nomc," nobmc = ",nobmc

       if (ecor .eq. 0) then
         allocate(r(nop,6),Jacob(nop,6),X(nop,12),Vabit(nop))
         allocate(Vfit(nop),V2b(nop))
         allocate(V2bAB(nop),V2bAC(nop),V2bAD(nop),V2bBC(nop),
     $            V2bBD(nop),V2bCD(nop),disper(nop))
       else if (ecor .eq. 1) then
         allocate(r(nop,6),Jacob(nop,6),X(nop,12),Vabit(nop),
     $         Vcas(nop),Vcor(nop))
         allocate(Vfit(nop),Vcasf(nop),Vcorf(nop),
     $          V2b(nop),V2bcas(nop),V2bcor(nop))
         allocate(V2bAB(nop),V2bAC(nop),V2bAD(nop),V2bBC(nop),
     $            V2bBD(nop),V2bCD(nop))
         allocate(V2bcorAB(nop),V2bcorAC(nop),V2bcorAD(nop),
     $            V2bcorBC(nop),V2bcorBD(nop),V2bcorCD(nop),
     $            disper(nop))
       else if (ecor .eq. 2) then
         allocate(r(nop,6),Jacob(nop,6),X(nop,12),Vcas(nop))
         allocate(Vcasf(nop),V2bcas(nop))
         allocate(V2bAB(nop),V2bAC(nop),V2bAD(nop),V2bBC(nop),
     $            V2bBD(nop),V2bCD(nop))
       else if (ecor .eq. 3) then
         allocate(rcr(nopc,6),Jacobc(nopc,6),Xc(nopc,12),Vcor(nopc))
         allocate(Vcorf(nopc),V2bcor(nopc))
         allocate(V2bcorAB(nopc),V2bcorAC(nopc),V2bcorAD(nopc),
     $            V2bcorBC(nopc),V2bcorBD(nopc),V2bcorCD(nopc),
     $            disper(nopc))
       else if (ecor .eq. 4) then
         allocate(r(nop,6),Jacob(nop,6),X(nop,12),Vcas(nop))
         allocate(rcr(nopc,6),Jacobc(nopc,6),Xc(nopc,12),Vcor(nopc))
         allocate(Vcasf(nop),Vcorf(nopc),V2bcas(nop),V2bcor(nopc))
         allocate(V2bAB(nop),V2bAC(nop),V2bAD(nop),V2bBC(nop),
     $            V2bBD(nop),V2bCD(nop))
         allocate(V2bcorAB(nopc),V2bcorAC(nopc),V2bcorAD(nopc),
     $            V2bcorBC(nopc),V2bcorBD(nopc),V2bcorCD(nopc),
     $            disper(nopc))
       endif

       if (ecor .eq. 0) then
         do j=1,nop
           do i=1,6
             r(j,i)=0.0d0
             Jacob(j,i)=0.0d0
           end do
           do i=1,12
             X(j,i)=0.0d0
           end do
           Vabit(j)=0.0d0
           Vfit(i)=0.0d0
           V2b(i)=0.0d0
           V2bAB(i)=0.0d0
           V2bAC(i)=0.0d0
           V2bAD(i)=0.0d0
           V2bBC(i)=0.0d0
           V2bBD(i)=0.0d0
           V2bCD(i)=0.0d0
           disper(i)=0.0d0
         end do
       else if (ecor .eq. 1) then
       do j=1,nop
         do i=1,6
           r(j,i)=0.0d0
           Jacob(j,i)=0.0d0
         end do
         do i=1,12
           X(j,i)=0.0d0
         end do
          Vabit(j)=0.0d0
          Vcas(j)=0.0d0
          Vcor(j)=0.0d0
          Vfit(i)=0.0d0
          Vcasf(i)=0.0d0
          Vcorf(i)=0.0d0
          V2b(i)=0.0d0
          V2bcas(i)=0.0d0
          V2bcor(i)=0.0d0
          V2bAB(i)=0.0d0
          V2bAC(i)=0.0d0
          V2bAD(i)=0.0d0
          V2bBC(i)=0.0d0
          V2bBD(i)=0.0d0
          V2bCD(i)=0.0d0
          V2bcorAB(i)=0.0d0
          V2bcorAC(i)=0.0d0
          V2bcorAD(i)=0.0d0
          V2bcorBC(i)=0.0d0
          V2bcorBD(i)=0.0d0
          V2bcorCD(i)=0.0d0
          disper(i)=0.0d0
         enddo
       else if (ecor .eq. 2) then
         do j=1,nop
           do i=1,6
             r(j,i)=0.0d0
             Jacob(j,i)=0.0d0
           end do
           do i=1,12
             X(j,i)=0.0d0
           end do
           Vcas(j)=0.0d0
           Vcasf(i)=0.0d0
           V2bcas(i)=0.0d0
           V2bAB(i)=0.0d0
           V2bAC(i)=0.0d0
           V2bAD(i)=0.0d0
           V2bBC(i)=0.0d0
           V2bBD(i)=0.0d0
           V2bCD(i)=0.0d0
         end do
       else if (ecor .eq. 3) then
         do j=1,nopc
           do i=1,6
             rcr(j,i)=0.0d0
             Jacobc(j,i)=0.0d0
           end do
           do i=1,12
             Xc(j,i)=0.0d0
           end do
           Vcor(j)=0.0d0
           Vcorf(i)=0.0d0
           V2bcor(i)=0.0d0
           V2bcorAB(i)=0.0d0
           V2bcorAC(i)=0.0d0
           V2bcorAD(i)=0.0d0
           V2bcorBC(i)=0.0d0
           V2bcorBD(i)=0.0d0
           V2bcorCD(i)=0.0d0
           disper(i)=0.0d0
         end do
       else if (ecor .eq. 4) then
         do j=1,nop
           do i=1,6
             r(j,i)=0.0d0
             Jacob(j,i)=0.0d0
           end do
           do i=1,12
             X(j,i)=0.0d0
           end do
          Vcas(j)=0.0d0
          Vcasf(j)=0.0d0
          V2bcas(i)=0.0d0
          V2bAB(i)=0.0d0
          V2bAC(i)=0.0d0
          V2bAD(i)=0.0d0
          V2bBC(i)=0.0d0
          V2bBD(i)=0.0d0
          V2bCD(i)=0.0d0
         end do
         do j=1,nopc
           do i=1,6
             rcr(j,i)=0.0d0
             Jacobc(j,i)=0.0d0
           end do
           do i=1,12
             Xc(j,i)=0.0d0
           end do
          Vcor(j)=0.0d0
          Vcorf(j)=0.0d0
          V2bcor(i)=0.0d0
          V2bcorAB(i)=0.0d0
          V2bcorAC(i)=0.0d0
          V2bcorAD(i)=0.0d0
          V2bcorBC(i)=0.0d0
          V2bcorBD(i)=0.0d0
          V2bcorCD(i)=0.0d0
          disper(i)=0.0d0
          end do
        endif

C Vx is a dummy vector for non-existing data
      allocate(Vx(nop))

      if (ecor .eq. 0) then
      call readdat(r,Jacob,X,Vabit,Vx,Vx,nop,pairw,ecor,ired,
     $             nred,0)
      else if (ecor .eq. 1) then
      call readdat(r,Jacob,X,Vabit,Vcas,Vcor,nop,pairw,ecor,ired,
     $             nred,0)
      else if (ecor .eq. 2) then 
      call readdat(r,Jacob,X,Vcas,Vx,Vx,nop,pairw,ecor,ired,
     $             nred,0)
      else if (ecor .eq. 3) then  
      call readdat(rcr,Jacobc,Xc,Vcor,Vx,Vx,nopc,pairw,ecor,ired,
     $             nred,1)
      else if (ecor .eq. 4) then  
      call readdat(r,Jacob,X,Vcas,Vx,Vx,nop,pairw,ecor,ired,
     $             nred,0)
      call readdat(rcr,Jacobc,Xc,Vcor,Vx,Vx,nopc,pairw,ecor,ired,
     $             nred,1)
      endif

C Generate matrix BM, BMcas, and BMcor as well as 
C vectors C, bma, bmb, etc.. based on nfun and ecor

       if (ecor .eq. 0) then
        if (nfun .eq. 1) then
          allocate(bma(nobm),BM(nop,nobm),C(nobm))
          do i=1,nobm
            bma(i) = 0.0d0
            C(i) = 0.0d0
            do j=1,nop
              BM(j,i) = 0.0d0
            enddo
          enddo
        else if (nfun .eq. 2) then
          allocate(bma(nobma),bmb(nobmb),BM(nop,nobm),C(nobm))
          do i=1,nobma
            bma(i) = 0.0d0
          enddo
          do i=1,nobmb
            bmb(i) = 0.0d0
          enddo
          do i=1,nobm 
            C(i) = 0.0d0
            do j=1,nop
              BM(j,i) = 0.0d0
            enddo
          enddo
        endif
       else if (ecor .eq. 1) then
        if (nfun .eq. 1) then
          allocate(bma(nobm),BMcas(nop,nobm),C(nobm))
          do i=1,nobm
            bma(i) = 0.0d0
            C(i) = 0.0d0
            do j=1,nop
              BMcas(j,i) = 0.0d0
            enddo
          enddo
        else if (nfun .eq. 2) then
          allocate(bma(nobma),bmb(nobmb),BMcas(nop,nobm),C(nobm))
          do i=1,nobma
            bma(i) = 0.0d0
          enddo
          do i=1,nobmb
            bmb(i) = 0.0d0
          enddo
          do i=1,nobm 
            C(i) = 0.0d0
            do j=1,nop
              BMcas(j,i) = 0.0d0
            enddo
          enddo
        endif
          allocate(bmc(nobmc),BMcor(nop,nobmc),Ccor(nobmc))
          do i=1,nobmc
            bmc(i) = 0.0d0
            Ccor(i) = 0.0d0
            do j=1,nop
              BMcor(j,i) = 0.0d0
            enddo
          enddo
       else if (ecor .eq. 2) then
       if (nfun .eq. 1) then
          allocate(bma(nobm),BMcas(nop,nobm),C(nobm))
          do i=1,nobm
            bma(i) = 0.0d0
            C(i) = 0.0d0
            do j=1,nop
              BMcas(j,i) = 0.0d0
            enddo
          enddo
        else if (nfun .eq. 2) then
          allocate(bma(nobma),bmb(nobmb),BMcas(nop,nobm),C(nobm))
          do i=1,nobma
            bma(i) = 0.0d0
          enddo
          do i=1,nobmb
            bmb(i) = 0.0d0
          enddo
          do i=1,nobm 
            C(i) = 0.0d0
            do j=1,nop
              BMcas(j,i) = 0.0d0
            enddo
          enddo
        endif
       else if (ecor .eq. 3) then
          allocate(bmc(nobmc),BMcor(nopc,nobmc),Ccor(nobmc))
          do i=1,nobmc
            bmc(i) = 0.0d0
            Ccor(i) = 0.0d0
            do j=1,nopc
              BMcor(j,i) = 0.0d0
            enddo
          enddo
       else if (ecor .eq. 4) then
        if (nfun .eq. 1) then
          allocate(bma(nobm),BMcas(nop,nobm),C(nobm))
          do i=1,nobm
            bma(i) = 0.0d0
            C(i) = 0.0d0
            do j=1,nop
              BMcas(j,i) = 0.0d0
            enddo
          enddo
        else if (nfun .eq. 2) then
          allocate(bma(nobma),bmb(nobmb),BMcas(nop,nobm),C(nobm))
          do i=1,nobma
            bma(i) = 0.0d0
          enddo
          do i=1,nobmb
            bmb(i) = 0.0d0
          enddo
          do i=1,nobm 
            C(i) = 0.0d0
            do j=1,nop
              BMcas(j,i) = 0.0d0
            enddo
          enddo
        endif
          allocate(bmc(nobmc),BMcor(nopc,nobmc),Ccor(nobmc))
          do i=1,nobmc
            bmc(i) = 0.0d0
            Ccor(i) = 0.0d0
            do j=1,nopc
              BMcor(j,i) = 0.0d0
            enddo
          enddo
       endif

C form basis function matrix BM

      if (ecor .eq. 0) then
        do i=1,nop
        do j=1,6
          dist(j)=r(i,j)
        enddo
       if (nfun .eq. 1) then
         call frmbm(dist,bma,iordera,noma,nobma,noba,1)
        if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
          do j=1,noba-1
            BM(i,j)=bma(j+1)
          enddo
          nob=noba-1
        else if (imethd .eq. 2) then
          do j=1,noba
            BM(i,j)=bma(j)
          enddo
          nob=noba
        else if (imethd .eq. 3) then
          do j=1,noba
            BM(i,j)=bma(j)
          enddo
          nob=noba
        endif
       else if (nfun .eq. 2) then
         call frmbm(dist,bma,iordera,noma,nobma,noba,1)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,noba-1
               BM(i,j)=bma(j+1)
             enddo
           else if (imethd .eq. 2) then
             do j=1,noba
               BM(i,j)=bma(j)
             enddo
           else if (imethd .eq. 3) then
             do j=1,noba
               BM(i,j)=bma(j)
             enddo
           endif
         call frmbm(dist,bmb,iorderb,nomb,nobmb,nobb,2)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,nobb-1
               BM(i,noba+j)=bmb(j+1)
             enddo
             nob=noba+nobb-2
           else if (imethd .eq. 2) then
             do j=1,nobb
               BM(i,noba+j)=bmb(j)
             enddo
             nob=noba+nobb
           else if (imethd .eq. 3) then
             do j=1,nobb
               BM(i,noba+j)=bmb(j)
             enddo
             nob=noba+nobb
           endif
       endif
       if (imethd .eq. 2) then
C Evaluate 2-body interactions and store in v2b(1:nop)
        call ev2b(dist,v,V2ABx,V2ACx,V2ADx,V2BCx,V2BDx,V2CDx,pairw
     $           ,disp)
           v2b(i)=v
C save the diatomic energies in vector format 
           V2bAB(i)=V2ABx   !+120.243d0
           V2bAC(i)=V2ACx   !+152.6d0
           V2bAD(i)=V2ADx   !+152.6d0
           V2bBC(i)=V2BCx   !+152.6d0
           V2bBD(i)=V2BDx   !+152.6d0
           V2bCD(i)=V2CDx   !+228.4d0
           disper(i)=disp
       endif
       enddo
      else if (ecor .eq. 1) then
        do i=1,nop
        do j=1,6
          dist(j)=r(i,j)
        enddo
       if (nfun .eq. 1) then
         call frmbm(dist,bma,iordera,noma,nobma,noba,1)
        if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
          do j=1,noba-1
            BMcas(i,j)=bma(j+1)
          enddo
          nob=noba-1
        else if (imethd .eq. 2) then
          do j=1,noba
            BMcas(i,j)=bma(j)
          enddo
          nob=noba
        else if (imethd .eq. 3) then
          do j=1,noba
            BMcas(i,j)=bma(j)
          enddo
          nob=noba
        endif
       else if (nfun .eq. 2) then
         call frmbm(dist,bma,iordera,noma,nobma,noba,1)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,noba-1
               BMcas(i,j)=bma(j+1)
             enddo
           else if (imethd .eq. 2) then
             do j=1,noba
               BMcas(i,j)=bma(j)
             enddo
           else if (imethd .eq. 3) then
             do j=1,noba
               BMcas(i,j)=bma(j)
             enddo
           endif
         call frmbm(dist,bmb,iorderb,nomb,nobmb,nobb,2)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,nobb-1
               BMcas(i,noba+j)=bmb(j+1)
             enddo
             nob=noba+nobb-2
           else if (imethd .eq. 2) then
             do j=1,nobb
               BMcas(i,noba+j)=bmb(j)
             enddo
             nob=noba+nobb
           else if (imethd .eq. 3) then
             do j=1,nobb
               BMcas(i,noba+j)=bmb(j)
             enddo
             nob=noba+nobb
           endif
       endif
       if (imethd .eq. 2) then
C Evaluate 2-body interactions and store in v2b(1:nop)
        call ev2bcas(dist,v,V2ABx,V2ACx,V2ADx,V2BCx,V2BDx,V2CDx,pairw)
           v2bcas(i)=v
C save the diatomic energies in vector format 
           V2bAB(i)=V2ABx  
           V2bAC(i)=V2ACx   
           V2bAD(i)=V2ADx   
           V2bBC(i)=V2BCx   
           V2bBD(i)=V2BDx   
           V2bCD(i)=V2CDx   
       endif
          call frmbm(dist,bmc,iorderc,nomc,nobmc,nobc,3)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,nobc-1
               BMcor(i,j)=bmc(j+1)
             enddo
             nobc=nobc-1
           else if (imethd .eq. 2) then
             do j=1,nobc
               BMcor(i,j)=bmc(j)
             enddo
           else if (imethd .eq. 3) then
             do j=1,nobc
               BMcor(i,j)=bmc(j)
             enddo
           endif
       if (imethd .eq. 2) then
C Evaluate 2-body interactions and store in v2bcas(1:nop)
        call ev2bcor(dist,v,V2ABx,V2ACx,V2ADx,V2BCx,V2BDx,V2CDx,pairw
     &              ,disp)
           v2bcor(i)=v
           v2b(i)=v2bcas(i) + v2bcor(i)
C save the diatomic energies in vector format 
           V2bcorAB(i)=V2ABx   
           V2bcorAC(i)=V2ACx   
           V2bcorAD(i)=V2ADx   
           V2bcorBC(i)=V2BCx   
           V2bcorBD(i)=V2BDx   
           V2bcorCD(i)=V2CDx
           disper(i)=disp
C get beck the Vtotal for diatomics
           V2bAB(i)=V2bAB(i) + V2bcorAB(i)  
           V2bAC(i)=V2bAC(i) + V2bcorAC(i)   
           V2bAD(i)=V2bAD(i) + V2bcorAD(i)   
           V2bBC(i)=V2bBC(i) + V2bcorBC(i)   
           V2bBD(i)=V2bBD(i) + V2bcorBD(i)   
           V2bCD(i)=V2bCD(i) + V2bcorCD(i)
       endif
       enddo
      else if (ecor .eq. 2) then
        do i=1,nop
        do j=1,6
          dist(j)=r(i,j)
        enddo
       if (nfun .eq. 1) then
         call frmbm(dist,bma,iordera,noma,nobma,noba,1)
        if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
          do j=1,noba-1
            BMcas(i,j)=bma(j+1)
          enddo
          nob=noba-1
        else if (imethd .eq. 2) then
          do j=1,noba
            BMcas(i,j)=bma(j)
          enddo
          nob=noba
        else if (imethd .eq. 3) then
          do j=1,noba
            BMcas(i,j)=bma(j)
          enddo
          nob=noba
        endif
       else if (nfun .eq. 2) then
         call frmbm(dist,bma,iordera,noma,nobma,noba,1)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,noba-1
               BMcas(i,j)=bma(j+1)
             enddo
           else if (imethd .eq. 2) then
             do j=1,noba
               BMcas(i,j)=bma(j)
             enddo
           else if (imethd .eq. 3) then
             do j=1,noba
               BMcas(i,j)=bma(j)
             enddo
           endif
         call frmbm(dist,bmb,iorderb,nomb,nobmb,nobb,2)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,nobb-1
               BMcas(i,noba+j)=bmb(j+1)
             enddo
             nob=noba+nobb-2
           else if (imethd .eq. 2) then
             do j=1,nobb
               BMcas(i,noba+j)=bmb(j)
             enddo
             nob=noba+nobb
           else if (imethd .eq. 3) then
             do j=1,nobb
               BMcas(i,noba+j)=bmb(j)
             enddo
             nob=noba+nobb
           endif
       endif
       if (imethd .eq. 2) then
C Evaluate 2-body interactions and store in v2bcas(1:nop)
        call ev2bcas(dist,v,V2ABx,V2ACx,V2ADx,V2BCx,V2BDx,V2CDx,pairw)
           v2bcas(i)=v
C save the diatomic energies in vector format 
           V2bAB(i)=V2ABx   
           V2bAC(i)=V2ACx   
           V2bAD(i)=V2ADx   
           V2bBC(i)=V2BCx   
           V2bBD(i)=V2BDx   
           V2bCD(i)=V2CDx   
       endif
       enddo
      else if (ecor .eq. 3) then
        do i=1,nopc
        do j=1,6
          distc(j)=rcr(i,j)
        enddo
         call frmbm(distc,bmc,iorderc,nomc,nobmc,nobc,3)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,nobc-1
               BMcor(i,j)=bmc(j+1)
             enddo
             nobc=nobc-1
           else if (imethd .eq. 2) then
             do j=1,nobc
               BMcor(i,j)=bmc(j)
             enddo
           else if (imethd .eq. 3) then
             do j=1,nobc
               BMcor(i,j)=bmc(j)
             enddo
           endif
       if (imethd .eq. 2) then
C Evaluate 2-body interactions and store in v2bcas(1:nop)
        call ev2bcor(distc,v,V2ABx,V2ACx,V2ADx,V2BCx,V2BDx,V2CDx,pairw
     &              ,disp)
           v2bcor(i)=v
C save the diatomic energies in vector format 
           V2bcorAB(i)=V2ABx   
           V2bcorAC(i)=V2ACx   
           V2bcorAD(i)=V2ADx   
           V2bcorBC(i)=V2BCx   
           V2bcorBD(i)=V2BDx   
           V2bcorCD(i)=V2CDx
           disper(i)=disp
       endif
       enddo
      else if (ecor .eq. 4) then
        do i=1,nop
        do j=1,6
          dist(j)=r(i,j)
        enddo
       if (nfun .eq. 1) then
         call frmbm(dist,bma,iordera,noma,nobma,noba,1)
        if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
          do j=1,noba-1
            BMcas(i,j)=bma(j+1)
          enddo
          nob=noba-1
        else if (imethd .eq. 2) then
          do j=1,noba
            BMcas(i,j)=bma(j)
          enddo
          nob=noba
        else if (imethd .eq. 3) then
          do j=1,noba
            BMcas(i,j)=bma(j)
          enddo
          nob=noba
        endif
       else if (nfun .eq. 2) then
         call frmbm(dist,bma,iordera,noma,nobma,noba,1)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,noba-1
               BMcas(i,j)=bma(j+1)
             enddo
           else if (imethd .eq. 2) then
             do j=1,noba
               BMcas(i,j)=bma(j)
             enddo
           else if (imethd .eq. 3) then
             do j=1,noba
               BMcas(i,j)=bma(j)
             enddo
           endif
         call frmbm(dist,bmb,iorderb,nomb,nobmb,nobb,2)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,nobb-1
               BMcas(i,noba+j)=bmb(j+1)
             enddo
             nob=noba+nobb-2
           else if (imethd .eq. 2) then
             do j=1,nobb
               BMcas(i,noba+j)=bmb(j)
             enddo
             nob=noba+nobb
           else if (imethd .eq. 3) then
             do j=1,nobb
               BMcas(i,noba+j)=bmb(j)
             enddo
             nob=noba+nobb
           endif
       endif
       if (imethd .eq. 2) then
C Evaluate 2-body interactions and store in v2b(1:nop)
        call ev2bcas(dist,v,V2ABx,V2ACx,V2ADx,V2BCx,V2BDx,V2CDx,pairw)
           v2bcas(i)=v
C save the diatomic energies in vector format 
           V2bAB(i)=V2ABx   
           V2bAC(i)=V2ACx   
           V2bAD(i)=V2ADx   
           V2bBC(i)=V2BCx   
           V2bBD(i)=V2BDx   
           V2bCD(i)=V2CDx   
       endif
       enddo
        do i=1,nopc
        do j=1,6
          distc(j)=rcr(i,j)
        enddo
         call frmbm(distc,bmc,iorderc,nomc,nobmc,nobc,3)
           if ((imethd .eq. 0) .or. (imethd .eq. 1)) then
             do j=1,nobc-1
               BMcor(i,j)=bmc(j+1)
             enddo
           else if (imethd .eq. 2) then
             do j=1,nobc
               BMcor(i,j)=bmc(j)
             enddo
           else if (imethd .eq. 3) then
             do j=1,nobc
               BMcor(i,j)=bmc(j)
             enddo
           endif
       if (imethd .eq. 2) then
C Evaluate 2-body interactions and store in v2bcas(1:nop)
        call ev2bcor(distc,v,V2ABx,V2ACx,V2ADx,V2BCx,V2BDx,V2CDx,pairw
     &               ,disp)
           v2bcor(i)=v
C save the diatomic energies in vector format 
           V2bcorAB(i)=V2ABx   
           V2bcorAC(i)=V2ACx   
           V2bcorAD(i)=V2ADx   
           V2bcorBC(i)=V2BCx   
           V2bcorBD(i)=V2BDx   
           V2bcorCD(i)=V2CDx
           disper(i)=disp
       endif
       enddo
      endif

C close the old arrays

      if (ecor .eq. 0) then
        if (nfun .eq. 1) then
          deallocate(bma)
        else if (nfun .eq. 2) then
          deallocate(bma,bmb)
        endif
      else if (ecor .eq. 1) then
        if (nfun .eq. 1) then
          deallocate(bma,bmc)
        else if (nfun .eq. 2) then
          deallocate(bma,bmb,bmc)
        endif
      else if (ecor .eq. 2) then
        if (nfun .eq. 1) then
          deallocate(bma)
        else if (nfun .eq. 2) then
          deallocate(bma,bmb)
        endif
      else if (ecor .eq. 3) then
          deallocate(bmc)
      else if (ecor .eq. 4) then
        if (nfun .eq. 1) then
          deallocate(bma,bmc)
        else if (nfun .eq. 2) then
          deallocate(bma,bmb,bmc)
        endif
      endif

C calculation of data closeness for weighing purpose (if iwop = 2)
      if (iwop .eq. 2) then
        if (ecor .ge. 3) then
        allocate (dvec(nop),dcvec(nopc),rsrt(nop,6),rcsrt(nopc,6))
          do i=1,nopc
          dcvec(i)=0.0d0
           do j=1,6
            rcsrt(i,j)=0.0d0
           enddo
          enddo
          do i=1,nopc
           do j=1,6
            distc(j)=rcr(i,j)
           enddo
            if (ityp .eq. 1) then
            call selsor1(distc)
            else if (ityp .eq. 3) then
            call selsor3(distc)
            endif
           do j=1,6
            rcsrt(i,j)=distc(j)
           enddo
          enddo
       do i=1,nopc
        do j=1,nopc
         dcvec(i)=dcvec(i) + sqrt((
     $  ((rcsrt(i,1)-rcsrt(j,1))/
     $  (rcsrt(i,1)+rcsrt(j,1)))**2.0d0+
     $  ((rcsrt(i,2)-rcsrt(j,2))/
     $  (rcsrt(i,2)+rcsrt(j,2)))**2.0d0+
     $  ((rcsrt(i,3)-rcsrt(j,3))/
     $  (rcsrt(i,3)+rcsrt(j,3)))**2.0d0+
     $  ((rcsrt(i,4)-rcsrt(j,4))/
     $  (rcsrt(i,4)+rcsrt(j,4)))**2.0d0+
     $  ((rcsrt(i,5)-rcsrt(j,5))/
     $  (rcsrt(i,5)+rcsrt(j,5)))**2.0d0+
     $  ((rcsrt(i,6)-rcsrt(j,6))/
     $  (rcsrt(i,6)+rcsrt(j,6)))**2.0d0)/6.0d0)/nopc
        enddo
       enddo
        dcvecm=maxval(dcvec(1:nop))
      do i=1,nop
        dcvec(i)=dcvec(i)/dcvecm
      enddo

        else if (ecor .le. 2) then
        allocate (dvec(nop),rsrt(nop,6))
        endif
        do i=1,nop
        dvec(i)=0.0d0
          do j=1,6
          rsrt(i,j)=0.0d0
          enddo
        enddo
          do i=1,nop
           do j=1,6
            dist(j)=r(i,j)
           enddo
            if (ityp .eq. 1) then
            call selsor1(dist)
            else if (ityp .eq. 3) then
            call selsor3(dist)
            endif
           do j=1,6
             rsrt(i,j)=dist(j)
           enddo
          enddo
       do i=1,nop
        do j=1,nop
         dvec(i)=dvec(i) + sqrt((
     $  ((rsrt(i,1)-rsrt(j,1))/
     $  (rsrt(i,1)+rsrt(j,1)))**2.0d0+
     $  ((rsrt(i,2)-rsrt(j,2))/
     $  (rsrt(i,2)+rsrt(j,2)))**2.0d0+
     $  ((rsrt(i,3)-rsrt(j,3))/
     $  (rsrt(i,3)+rsrt(j,3)))**2.0d0+
     $  ((rsrt(i,4)-rsrt(j,4))/
     $  (rsrt(i,4)+rsrt(j,4)))**2.0d0+
     $  ((rsrt(i,5)-rsrt(j,5))/
     $  (rsrt(i,5)+rsrt(j,5)))**2.0d0+
     $  ((rsrt(i,6)-rsrt(j,6))/
     $  (rsrt(i,6)+rsrt(j,6)))**2.0d0)/6.0d0)/nop
        enddo
       enddo
        dvecm=maxval(dvec(1:nop))
      do i=1,nop
        dvec(i)=dvec(i)/dvecm
      enddo
      close(91)
      endif

C write the coefficients into files
      if (ecor .eq. 0) then
       open(unit=40,file=trim(adjustl(cffu)))
      else if (ecor .eq. 1) then
       open(unit=40,file=trim(adjustl(cffucas)))
       open(unit=45,file=trim(adjustl(cffucor)))
      else if (ecor .eq. 2) then
       open(unit=40,file=trim(adjustl(cffucas)))
      else if (ecor .eq. 3) then
       open(unit=45,file=trim(adjustl(cffucor)))
      else if (ecor .eq. 4) then
       open(unit=40,file=trim(adjustl(cffucas)))
       open(unit=45,file=trim(adjustl(cffucor)))
      endif

      if (ecor .eq. 0) then
        read(40,*)
        do i=1,nob
        read(40,'(a)')kk
        iblnk=index(kk,' / ')
        read(kk(iblnk+2:),*) C(i)
        enddo
      else if (ecor .eq. 1) then
        read(40,*)
        do i=1,nob
        read(40,'(a)')kk
        iblnk=index(kk,' / ')
        read(kk(iblnk+2:),*) C(i)
        enddo
        read(45,*)
        do i=1,nobc
        read(45,'(a)')kk
        iblnk=index(kk,' / ')
        read(kk(iblnk+2:),*) Ccor(i)
        enddo
      else if (ecor .eq. 2) then
        read(40,*)
        do i=1,nob
        read(40,'(a)')kk
        iblnk=index(kk,' / ')
        read(kk(iblnk+2:),*) C(i)
        enddo
      else if (ecor .eq. 3) then
        read(45,*)
        do i=1,nobc
        read(45,'(a)')kk
        iblnk=index(kk,' / ')
        read(kk(iblnk+2:),*) Ccor(i)
        enddo
      else if (ecor .eq. 4) then
        read(40,*)
        do i=1,nob
        read(40,'(a)')kk
        iblnk=index(kk,' / ')
        read(kk(iblnk+2:),*) C(i)
        enddo
        read(45,*)
        do i=1,nobc
        read(45,'(a)')kk
        iblnk=index(kk,' / ')
        read(kk(iblnk+2:),*) Ccor(i)
        enddo
      endif

      if (ecor .eq. 0) then
      write(30,*) "The optimized linear coefficients has been 
     $ read from file ",trim(adjustl(cffu)),"."
      else if (ecor .eq. 1) then
      write(30,*) "The optimized linear coefficients of Ell has been
     $ read from file ",trim(adjustl(cffucas)),"."
      write(30,*) "The optimized linear coefficients of Ehl has been
     $ read from file ",trim(adjustl(cffucor)),"."
      close(unit=40)
      close(unit=45)
      else if (ecor .eq. 2) then
      write(30,*) "The optimized linear coefficients of Ell has been
     $ read from file ",trim(adjustl(cffucas)),"."
      else if (ecor .eq. 3) then
      write(30,*) "The optimized linear coefficients of Ehl has been
     $ read from file ",trim(adjustl(cffucor)),"."
      else if (ecor .eq. 4) then
      write(30,*) "The optimized linear coefficients of Ell has been
     $ read from file ",trim(adjustl(cffucas)),"."
      write(30,*) "The optimized linear coefficients of Ehl has been
     $ read from file ",trim(adjustl(cffucor)),"."
      close(unit=40)
      close(unit=45)
      endif

C
C Use BM and C to evaluate the potential energies
C
      if (ecor .eq. 0) then
        call matvect(BM,C,Vfit,nop,nobm,nob) 
      else if (ecor .eq. 1) then
        call matvect(BMcas,C,Vcasf,nop,nobm,nob)
        call matvect(BMcor,Ccor,Vcorf,nop,nobmc,nobc)
          do i=1,nop
            vfit(i)=vcasf(i)+vcorf(i)
          enddo
      else if (ecor .eq. 2) then
        call matvect(BMcas,C,Vcasf,nop,nobm,nob)
      else if (ecor .eq. 3) then
        call matvect(BMcor,Ccor,Vcorf,nopc,nobmc,nobc)
      else if (ecor .eq. 4) then
        call matvect(BMcas,C,Vcasf,nop,nobm,nob)
        call matvect(BMcor,Ccor,Vcorf,nopc,nobmc,nobc)
      endif

      if (ecor .eq. 1) then 
      allocate(Vcasfx(nop),Vcorfx(nop))
      do i=1,nop
        Vcasfx(i)=vcasf(i)
        Vcorfx(i)=vcorf(i)
      enddo
      endif

C this part adds back the diatomic energies
      if (imethd .eq. 2) then
        if (ecor .eq. 0) then
            do i=1,nop
              vfit(i)=vfit(i)+v2b(i)
            enddo 
        else if (ecor .eq. 1) then
            do i=1,nop
              vfit(i)=vfit(i)+v2b(i)
            enddo
        else if (ecor .eq. 2) then
            do i=1,nop
              vcasf(i)=vcasf(i)+v2bcas(i)
            enddo
        else if (ecor .eq. 3) then
            do i=1,nopc
             vcorf(i)=vcorf(i)+v2bcor(i)
            enddo
        else if (ecor .eq. 4) then
            do i=1,nop
              vcasf(i)=vcasf(i)+v2bcas(i)
            enddo
            do i=1,nopc
             vcorf(i)=vcorf(i)+v2bcor(i)
            enddo
        endif
      endif

C write out the results
C 1. energy files

      if (ecor .eq. 0) then
        open(unit=50,file=trim(adjustl(vfu)))
         write(50,4201) nop
         if (icoord .eq. 0) then
          write(50,*) 'Index    rA        rB         d      thetaA
     $     thetaB    phi   Vabit     Vfit' 
          do i=1,nop
            write(50,4202) i,Jacob(i,1),Jacob(i,2),Jacob(i,3),
     $                     Jacob(i,4),Jacob(i,5),Jacob(i,6),
     $                     Vabit(i),Vfit(i)   
          enddo
         else if (icoord .eq. 1) then
          write(50,*) 'Index    Vabit          Vfit'
          write(50,*) 'X(1)   Y(1)   Z(1)'
          write(50,*) 'X(2)   Y(2)   Z(2)'
          write(50,*) 'X(3)   Y(3)   Z(3)'
          write(50,*) 'X(4)   Y(4)   Z(4)'
          write(50,*) ""
          do i=1,nop
            write(50,4203) i,Vabit(i),Vfit(i)
            write(50,4204) X(i,1),X(i,2),X(i,3)
            write(50,4204) X(i,4),X(i,5),X(i,6)
            write(50,4204) X(i,7),X(i,8),X(i,9)
            write(50,4204) X(i,10),X(i,11),X(i,12)
          enddo  
         else if (icoord .eq. 2) then
          write(50,*) 'Index    r1        r2       r3     r4
     $     r5     r6   Vabit     Vfit'
          do i=1,nop
            write(50,4202) i,r(i,1),r(i,2),r(i,3),
     $                     r(i,4),r(i,5),r(i,6),
     $                     Vabit(i),Vfit(i)
          enddo
         endif
        close(50)
C Write .vf1 file mainly for diatomic energies -VZ     
       if (iout .ge. 1) then
        open(unit=60,file=trim(adjustl(vfeu)))
          write(60,fmt='(a88)') "   i, Vabit, Vfit, V2bAB, V2bAC,
     $ V2bAD, V2bBC, V2bBD, V2bCD, where ABCD mean the atoms, disp"
        do i=1,nop
          write(60,4205) i,Vabit(i),Vfit(i),V2bAB(i),V2bAC(i),
     $                   V2bAD(i),V2bBC(i),V2bBD(i),V2bCD(i),
     $                   disper(i)
        enddo
        close(60)
       endif
C Write .vf2 file for terms of the multinomial -VZ
       if (iout .eq. 2) then
        open(unit=70,file=trim(adjustl(vfe2u)))
          write(70,*) "   i, Vabit, Vfit, and multnomial variables" 
        do i=1,nop
          write(70,4203) i,Vabit(i),Vfit(i)
         do j=1,nob
          write(70,fmt='(E25.15E3)') BM(i,j)
         enddo
        enddo
        close(70)
       endif
       write(30,*) "The fitted and ab initio energies for each",
     $  " geometry is written into ",trim(adjustl(vfu)),"file."
      else if (ecor .eq. 1) then
        open(unit=50,file=trim(adjustl(vfu)))
         write(50,4201) nop
         if (icoord .eq. 0) then
          write(50,*) 'Index    rA        rB         d      thetaA
     $     thetaB    phi   Vabit     Vfit' 
          do i=1,nop
            write(50,4202) i,Jacob(i,1),Jacob(i,2),Jacob(i,3),
     $                     Jacob(i,4),Jacob(i,5),Jacob(i,6),
     $                     Vabit(i),Vfit(i)   
          enddo
         else if (icoord .eq. 1) then
          write(50,*) 'Index    Vabit          Vfit'
          write(50,*) 'X(1)   Y(1)   Z(1)'
          write(50,*) 'X(2)   Y(2)   Z(2)'
          write(50,*) 'X(3)   Y(3)   Z(3)'
          write(50,*) 'X(4)   Y(4)   Z(4)'
          write(50,*) ""
          do i=1,nop
            write(50,4203) i,Vabit(i),Vfit(i)
            write(50,4204) X(i,1),X(i,2),X(i,3)
            write(50,4204) X(i,4),X(i,5),X(i,6)
            write(50,4204) X(i,7),X(i,8),X(i,9)
            write(50,4204) X(i,10),X(i,11),X(i,12)
          enddo  
         else if (icoord .eq. 2) then
          write(50,*) 'Index    r1        r2       r3     r4
     $     r5     r6   Vabit     Vfit'
          do i=1,nop
            write(50,4202) i,r(i,1),r(i,2),r(i,3),
     $                     r(i,4),r(i,5),r(i,6),
     $                     Vabit(i),Vfit(i)
          enddo
         endif
        close(50)
C Write .vf1 file mainly for diatomic energies -VZ     
       if (iout .ge. 1) then
        open(unit=60,file=trim(adjustl(vfeu)))
          write(60,fmt='(a88)') "   i, Vabit, Vfit, V2bAB, V2bAC,
     $ V2bAD, V2bBC, V2bBD, V2bCD, where ABCD mean the atoms, disp"
        do i=1,nop
          write(60,4205) i,Vabit(i),Vfit(i),V2bAB(i),V2bAC(i),
     $                   V2bAD(i),V2bBC(i),V2bBD(i),V2bCD(i),
     $                   disper(nop)
        enddo
        close(60)
       endif
       write(30,*) "The fitted and ab initio energies for each",
     $  " geometry is written into several files."
      open(unit=80,file=(trim(adjustl(vfcas))))
          write(80,*) "Vll part of the fit"
          write(80,*) "i, Vll, Vllf, and diatomics" 
        do i=1,nop
          write(80,7000) i,Vcas(i),Vcasfx(i)+V2bcas(i),
     $    V2bAB(i)-V2bcorAB(i),V2bAC(i)-V2bcorAC(i),
     $    V2bAD(i)-V2bcorAD(i),V2bBC(i)-V2bcorBC(i),
     $    V2bBD(i)-V2bcorBD(i),V2bCD(i)-V2bcorCD(i) 
         enddo
      close(80)
       open(unit=81,file=trim(adjustl(vfcor)))
          write(81,*) "Vhl part of the fit"
          write(81,*) "i, Vhl, Vhlf, and diatomics, disp"
        do i=1,nop
          write(81,7001) i,Vcor(i),Vcorfx(i)+V2bcor(i),
     $    V2bcorAB(i),V2bcorAC(i),V2bcorAD(i),V2bcorBC(i),V2bcorBD(i),
     $    V2bcorCD(i),disper(i)
        enddo
       close(81)

      else if (ecor .eq. 2) then
        open(unit=50,file=trim(adjustl(vfu)))
         write(50,4201) nop
         if (icoord .eq. 0) then
          write(50,*) 'Index    rA        rB         d      thetaA
     $     thetaB    phi   Vll     Vllf' 
          do i=1,nop
            write(50,4202) i,Jacob(i,1),Jacob(i,2),Jacob(i,3),
     $                     Jacob(i,4),Jacob(i,5),Jacob(i,6),
     $                     Vcas(i),Vcasf(i)   
          enddo
         else if (icoord .eq. 1) then
          write(50,*) 'Index    Vll          Vllf'
          write(50,*) 'X(1)   Y(1)   Z(1)'
          write(50,*) 'X(2)   Y(2)   Z(2)'
          write(50,*) 'X(3)   Y(3)   Z(3)'
          write(50,*) 'X(4)   Y(4)   Z(4)'
          write(50,*) ""
          do i=1,nop
            write(50,4203) i,Vcas(i),Vcasf(i)
            write(50,4204) X(i,1),X(i,2),X(i,3)
            write(50,4204) X(i,4),X(i,5),X(i,6)
            write(50,4204) X(i,7),X(i,8),X(i,9)
            write(50,4204) X(i,10),X(i,11),X(i,12)
          enddo  
         else if (icoord .eq. 2) then
          write(50,*) 'Index    r1        r2       r3     r4
     $     r5     r6   Vll     Vllf'
          do i=1,nop
            write(50,4202) i,r(i,1),r(i,2),r(i,3),
     $                     r(i,4),r(i,5),r(i,6),
     $                     Vcas(i),Vcasf(i)
          enddo
         endif
        close(50)
C Write .vf1 file mainly for diatomic energies -VZ     
       if (iout .ge. 1) then
        open(unit=60,file=trim(adjustl(vfeu)))
          write(60,fmt='(a86)') "   i, Vll, Vllf, V2bAB, V2bAC,
     $ V2bAD, V2bBC, V2bBD, V2bCD, where ABCD mean the atoms, disp"
        do i=1,nop
          write(60,4205) i,Vcas(i),Vcasf(i),V2bAB(i),V2bAC(i),
     $                    V2bAD(i),V2bBC(i),V2bBD(i),V2bCD(i),
     $                    disper(i)
        enddo
        close(60)
       endif
C Write .vf2 file for terms of the multinomial -VZ
       if (iout .eq. 2) then
        open(unit=70,file=trim(adjustl(vfe2u)))
          write(70,*) "   i, Vll, Vllf, and multnomial variables" 
        do i=1,nop
          write(70,4203) i,Vcas(i),Vcasf(i)
         do j=1,nob
          write(70,fmt='(E25.15E3)') BMcas(i,j)
         enddo
        enddo
        close(70)
       endif
       write(30,*) "The fitted and Vll energies for each",
     $  " geometry is written into ",trim(adjustl(vfu)),"file."
      else if (ecor .eq. 3) then
        open(unit=50,file=trim(adjustl(vfu)))
         write(50,4201) nopc
         if (icoord .eq. 0) then
          write(50,*) 'Index    rA        rB         d      thetaA
     $     thetaB    phi   Vhl     Vhlf' 
          do i=1,nopc
            write(50,4202) i,Jacobc(i,1),Jacobc(i,2),Jacobc(i,3),
     $                     Jacobc(i,4),Jacobc(i,5),Jacobc(i,6),
     $                     Vcor(i),Vcorf(i)   
          enddo
         else if (icoord .eq. 1) then
          write(50,*) 'Index    Vhl          Vhlf'
          write(50,*) 'X(1)   Y(1)   Z(1)'
          write(50,*) 'X(2)   Y(2)   Z(2)'
          write(50,*) 'X(3)   Y(3)   Z(3)'
          write(50,*) 'X(4)   Y(4)   Z(4)'
          write(50,*) ""
          do i=1,nopc
            write(50,4203) i,Vcor(i),Vcorf(i)
            write(50,4204) Xc(i,1),Xc(i,2),Xc(i,3)
            write(50,4204) Xc(i,4),Xc(i,5),Xc(i,6)
            write(50,4204) Xc(i,7),Xc(i,8),Xc(i,9)
            write(50,4204) Xc(i,10),Xc(i,11),Xc(i,12)
          enddo  
         else if (icoord .eq. 2) then
          write(50,*) 'Index    r1        r2       r3     r4
     $     r5     r6   Vhl     Vhlf'
          do i=1,nop
            write(50,4202) i,r(i,1),r(i,2),r(i,3),
     $                     r(i,4),r(i,5),r(i,6),
     $                     Vcor(i),Vcorf(i)
          enddo

         endif
        close(50)
C Write .vf1 file mainly for diatomic energies -VZ     
       if (iout .ge. 1) then
        open(unit=60,file=trim(adjustl(vfeu)))
          write(60,fmt='(a86)') "   i, Vhl, Vhlf, V2bAB, V2bAC,
     $ V2bAD, V2bBC, V2bBD, V2bCD, where ABCD mean the atoms, disp"
        do i=1,nopc
          write(60,4205) i,Vcor(i),Vcorf(i),V2bcorAB(i),V2bcorAC(i),
     $                   V2bcorAD(i),V2bcorBC(i),V2bcorBD(i),
     $                   V2bcorCD(i),disper(i)
        enddo
        close(60)
       endif
C Write .vf2 file for terms of the multinomial -VZ
       if (iout .eq. 2) then
        open(unit=70,file=trim(adjustl(vfe2u)))
          write(70,*) "   i, Vhl, Vhlf, and multinomial variables" 
        do i=1,nopc
          write(70,4203) i,Vcor(i),Vcorf(i)
         do j=1,nobc
          write(70,fmt='(E25.15E3)') BMcor(i,j)
         enddo
        enddo
        close(70)
       endif
       write(30,*) "The fitted and Vhl energies for each",
     $  " geometry is written into ",trim(adjustl(vfu))," file."
      else if (ecor .eq. 4) then
      open(unit=80,file=trim(adjustl(vfcas)))
          write(80,*) "Geometries for Vll"
          write(80,*) "i, Vll, Vllf, and diatomics" 
        do i=1,nop
          write(80,7000) i,Vcas(i),Vcasf(i),
     $    V2bAB(i),V2bAC(i),V2bAD(i),V2bBC(i),V2bBD(i),V2bCD(i) 
         enddo
      close(80)
       open(unit=81,file=trim(adjustl(vfcor)))
          write(81,*) "Geometries of Vhl"
          write(81,*) "i, Vhl, Vhlf, and diatomics, disp"
        do i=1,nopc
          write(81,7001) i,Vcor(i),Vcorf(i),
     $    V2bcorAB(i),V2bcorAC(i),V2bcorAD(i),V2bcorBC(i),V2bcorBD(i),
     $    V2bcorCD(i),disper(i)
        enddo
       close(81)
      write(30,*) "The fitted and ab initio energies for each geometry
     $ is written into files ",trim(adjustl(vfcas))," and ",
     $ trim(adjustl(vfcor)),"."
      endif

 4201 Format('Number of data points:',I8)
 4202 Format(1x,I6,3f10.5,3f8.3,2f10.2)
 4203 Format(1X,I6,2f12.5)
 4204 Format(1X,3f18.10)
 4205 Format(1X,I6,9f12.5)
 7000 Format(1X,I6,8f12.5)
 7001 Format(1X,I6,9f12.5)

      if (ecor .eq. 0) then
       write(30,*)
       write(30,*) "Error analysis for Vtotal based on energy:"
       write(30,5401) erange(1)
       write(30,5402) erange(1),erange(2)
       write(30,5403) erange(2),erange(3)
       write(30,5404) erange(3),erange(4)
       write(30,5405) erange(4)
       write(30,*)
       call calrmsd(Vabit,Vfit,erange,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5406)i,nops(i),i,rmsd(i)
       enddo
       write(30,5406) 6,nops(6),6,rmsd(6)

       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5407) i,nops(i),i,mud(i)
       enddo

       write(30,*)
       write(30,*) "Error analysis for Vtotal based on geometry:"
       write(30,5411) grange(1)
       write(30,5412) grange(1),grange(2)
       write(30,5413) grange(2),grange(3)
       write(30,5414) grange(3),grange(4)
       write(30,5415) grange(4)
       write(30,*)
       call calrmsd_geom(Vabit,Vfit,grange,r,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5416)i,nops(i),i,rmsd(i)
       enddo
       write(30,5416) 6,nops(6),6,rmsd(6)

       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5417) i,nops(i),i,mud(i)
       enddo

      else if (ecor .eq. 1) then
       write(30,*)
       write(30,*) "Error analysis for Vtotal based on energy:"
       write(30,5401) erange(1)
       write(30,5402) erange(1),erange(2)
       write(30,5403) erange(2),erange(3)
       write(30,5404) erange(3),erange(4)
       write(30,5405) erange(4)
       write(30,*)
       call calrmsd(Vabit,Vfit,erange,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5406)i,nops(i),i,rmsd(i)
       enddo
       write(30,5406) 6,nops(6),6,rmsd(6)

       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5407) i,nops(i),i,mud(i)
       enddo
       write(30,*)
       write(30,*) "Error analysis for Vll based on energy:"
       write(30,5401) erange(1)
       write(30,5402) erange(1),erange(2)
       write(30,5403) erange(2),erange(3)
       write(30,5404) erange(3),erange(4)
       write(30,5405) erange(4)
       write(30,*)

       do i=1,nop
       Vcasf(i)=Vcasfx(i)+V2bcas(i)
       enddo

       call calrmsd(Vcas,Vcasf,erange,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5406)i,nops(i),i,rmsd(i)
       enddo
       write(30,5406) 6,nops(6),6,rmsd(6)
       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5407) i,nops(i),i,mud(i)
       enddo

         erange(1)= -50.0d0
         erange(2)= -25.0d0
         erange(3)=   0.0d0
         erange(4)=  25.0d0
       write(30,*)
       write(30,*) "Error analysis for Vhl based on energy:"
       write(30,5401) erange(1)
       write(30,5402) erange(1),erange(2)
       write(30,5403) erange(2),erange(3)
       write(30,5404) erange(3),erange(4)
       write(30,5405) erange(4)
       write(30,*)

       do i=1,nop
       Vcorf(i)=Vcorfx(i)+V2bcor(i)
       enddo

       call calrmsd(Vcor,Vcorf,erange,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5406)i,nops(i),i,rmsd(i)
       enddo
       write(30,5406) 6,nops(6),6,rmsd(6)
       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5407) i,nops(i),i,mud(i)
       enddo
       write(30,*)
       write(30,*) "Error analysis for Vtotal based on geometry:"
       write(30,5411) grange(1)
       write(30,5412) grange(1),grange(2)
       write(30,5413) grange(2),grange(3)
       write(30,5414) grange(3),grange(4)
       write(30,5415) grange(4)
       write(30,*)
       call calrmsd_geom(Vabit,Vfit,grange,r,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5416)i,nops(i),i,rmsd(i)
       enddo
       write(30,5416) 6,nops(6),6,rmsd(6)

       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5417) i,nops(i),i,mud(i)
       enddo

       write(30,*)
       write(30,*) "Error analysis for Vll based on geometry:"
       write(30,5411) grange(1)
       write(30,5412) grange(1),grange(2)
       write(30,5413) grange(2),grange(3)
       write(30,5414) grange(3),grange(4)
       write(30,5415) grange(4)
       write(30,*)

       call calrmsd_geom(Vcas,Vcasf,grange,r,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5416)i,nops(i),i,rmsd(i)
       enddo
       write(30,5416) 6,nops(6),6,rmsd(6)
       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5417) i,nops(i),i,mud(i)
       enddo

       write(30,*)
       write(30,*) "Error analysis for Vhl based on geometry:"
       write(30,5411) grange(1)
       write(30,5412) grange(1),grange(2)
       write(30,5413) grange(2),grange(3)
       write(30,5414) grange(3),grange(4)
       write(30,5415) grange(4)
       write(30,*)

       call calrmsd_geom(Vcor,Vcorf,grange,r,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5416)i,nops(i),i,rmsd(i)
       enddo
       write(30,5416) 6,nops(6),6,rmsd(6)
       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5417) i,nops(i),i,mud(i)
       enddo

      else if (ecor .eq. 2) then
       write(30,*)
       write(30,*) "Error analysis for Vll based on energy:"
       write(30,5401) erange(1)
       write(30,5402) erange(1),erange(2)
       write(30,5403) erange(2),erange(3)
       write(30,5404) erange(3),erange(4)
       write(30,5405) erange(4)
       write(30,*)
       call calrmsd(Vcas,Vcasf,erange,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5406)i,nops(i),i,rmsd(i)
       enddo
       write(30,5406) 6,nops(6),6,rmsd(6)
       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5407) i,nops(i),i,mud(i)
       enddo

       write(30,*)
       write(30,*) "Error analysis for Vll based on geometry:"
       write(30,5411) grange(1)
       write(30,5412) grange(1),grange(2)
       write(30,5413) grange(2),grange(3)
       write(30,5414) grange(3),grange(4)
       write(30,5415) grange(4)
       write(30,*)
       call calrmsd_geom(Vcas,Vcasf,grange,r,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5416)i,nops(i),i,rmsd(i)
       enddo
       write(30,5416) 6,nops(6),6,rmsd(6)

       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5417) i,nops(i),i,mud(i)
       enddo

      else if (ecor .eq. 3) then
         erange(1)= -50.0d0
         erange(2)= -25.0d0
         erange(3)=   0.0d0
         erange(4)=  25.0d0
       write(30,*)
       write(30,*) "Error analysis for Vhl based on energy:"
       write(30,5401) erange(1)
       write(30,5402) erange(1),erange(2)
       write(30,5403) erange(2),erange(3)
       write(30,5404) erange(3),erange(4)
       write(30,5405) erange(4)
       write(30,*)
       call calrmsd(Vcor,Vcorf,erange,nopc,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5406)i,nops(i),i,rmsd(i)
       enddo
       write(30,5406) 6,nops(6),6,rmsd(6)
       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5407) i,nops(i),i,mud(i)
       enddo

       write(30,*)
       write(30,*) "Error analysis for Vhl based on geometry:"
       write(30,5411) grange(1)
       write(30,5412) grange(1),grange(2)
       write(30,5413) grange(2),grange(3)
       write(30,5414) grange(3),grange(4)
       write(30,5415) grange(4)
       write(30,*)
       call calrmsd_geom(Vcor,Vcorf,grange,rcr,nopc,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5416)i,nops(i),i,rmsd(i)
       enddo
       write(30,5416) 6,nops(6),6,rmsd(6)

       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5417) i,nops(i),i,mud(i)
       enddo

      else if (ecor .eq. 4) then
       write(30,*)
       write(30,*) "Error analysis for Vll:"
       write(30,5401) erange(1)
       write(30,5402) erange(1),erange(2)
       write(30,5403) erange(2),erange(3)
       write(30,5404) erange(3),erange(4)
       write(30,5405) erange(4)
       write(30,*)
       call calrmsd(Vcas,Vcasf,erange,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5406)i,nops(i),i,rmsd(i)
       enddo
       write(30,5406) 6,nops(6),6,rmsd(6)
       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5407) i,nops(i),i,mud(i)
       enddo
         erange(1)= -50.0d0
         erange(2)= -25.0d0
         erange(3)=   0.0d0
         erange(4)=  25.0d0
       write(30,*)
       write(30,*) "Error analysis for Vhl:"
       write(30,5401) erange(1)
       write(30,5402) erange(1),erange(2)
       write(30,5403) erange(2),erange(3)
       write(30,5404) erange(3),erange(4)
       write(30,5405) erange(4)
       write(30,*)
       call calrmsd(Vcor,Vcorf,erange,nopc,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5406)i,nops(i),i,rmsd(i)
       enddo
       write(30,5406) 6,nops(6),6,rmsd(6)
       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5407) i,nops(i),i,mud(i)
       enddo

       write(30,*)
       write(30,*) "Error analysis for Vll based on geometry:"
       write(30,5411) grange(1)
       write(30,5412) grange(1),grange(2)
       write(30,5413) grange(2),grange(3)
       write(30,5414) grange(3),grange(4)
       write(30,5415) grange(4)
       write(30,*)
       call calrmsd_geom(Vcas,Vcorf,grange,r,nop,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5416)i,nops(i),i,rmsd(i)
       enddo
       write(30,5416) 6,nops(6),6,rmsd(6)

       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5417) i,nops(i),i,mud(i)
       enddo

       write(30,*)
       write(30,*) "Error analysis for Vhl based on geometry:"
       write(30,5411) grange(1)
       write(30,5412) grange(1),grange(2)
       write(30,5413) grange(2),grange(3)
       write(30,5414) grange(3),grange(4)
       write(30,5415) grange(4)
       write(30,*)
       call calrmsd_geom(Vcor,Vcorf,grange,rcr,nopc,nops,rmsd,sd,mud)
       do i=1,5
         write(30,5416)i,nops(i),i,rmsd(i)
       enddo
       write(30,5416) 6,nops(6),6,rmsd(6)

       write(30,*) 'Squared deviation',sd(6)
       do i=1,6
         write(30,5417) i,nops(i),i,mud(i)
       enddo

      endif

 5401 Format(' Energy range 1: Vabit < ',F6.1,' kcal/mol')
 5402 Format(' Energy range 2: ',F6.1,' <= Vabit < ',F6.1,' kcal/mol')
 5403 Format(' Energy range 3: ',F6.1,' <= Vabit < ',F6.1,' kcal/mol')
 5404 Format(' Energy range 4: ',F6.1,' <= Vabit < ',F6.1,' kcal/mol')
 5405 Format(' Energy range 5: Vabit >= ',F6.1,' kcal/mol')
 5406 Format(' NOPs',I1,' = ',I6,' RMSD for energy range',I2,' is',
     $ F10.2,' kcal/mol')
 5407 Format(' NOPs',I1,' = ',I6,'  MUD for energy range',I2,' is',
     $ F10.2,' kcal/mol')

 5411 Format(' Geometry range 1: dist < ',F6.1,' Ang')
 5412 Format(' Geometry range 2: ',F6.1,' <= dist < ',F6.1,' Ang')
 5413 Format(' Geometry range 3: ',F6.1,' <= dist < ',F6.1,' Ang')
 5414 Format(' Geometry range 4: ',F6.1,' <= dist < ',F6.1,' Ang')
 5415 Format(' Geometry range 5: dist >= ',F6.1,' Ang')
 5416 Format(' NOPs',I1,' = ',I6,' RMSD for geometry range',I2,' is',
     $ F10.2,' kcal/mol')
 5417 Format(' NOPs',I1,' = ',I6,'  MUD for geometry range',I2,' is',
     $ F10.2,' kcal/mol')


C
C Call Sort subroutine to sort data points from largest absolute error
C to smallest 
C
       if (ecor .eq. 0) then
       call sort(Vabit,Vfit,Jacob,X,r,nop)
       write(30,*)
       write(30,6301) trim(adjustl(erru)),sael
       write(30,*) "will be listed for points with ab initio energies"
       write(30,6302) smiel,smael
       write(30,*)
       else if (ecor .eq. 1) then
       call sort(Vabit,Vfit,Jacob,X,r,nop)
       write(30,*)
       write(30,6301) trim(adjustl(erru)),sael
       write(30,*) "will be listed for points with ab initio energies"
       write(30,6302) smiel,smael
       write(30,*)
       else if (ecor .eq. 2) then
       call sort(Vcas,Vcasf,Jacob,X,r,nop)
       write(30,*)
       write(30,6301) trim(adjustl(erru)),sael
       write(30,*) "will be listed for points with ab initio energies"
       write(30,6302) smiel,smael
       write(30,*)
       else if (ecor .eq. 2) then
       call sort(Vcor,Vcorf,Jacobc,Xc,rcr,nopc)
       write(30,*)
       write(30,6301) trim(adjustl(erru)),sael
       write(30,*) "will be listed for points with ab initio energies"
       write(30,6302) smiel,smael
       write(30,*)
C       else if (ecor .eq. 4) then
C      it is not used yet
       endif

 6301 Format(" In ",a," file, errors larger than",f6.1," kcal/mol" )
 6302 Format(" between ",f6.1," and ",f6.1," (in kcal/mol)" )

C Do a statistical analysis for the coeffs 
C if ISAN=1       -VZ 043014
      if (isan .eq. 1) then
      if (ecor .eq. 0) then
        allocate(Cab(nob))
        do i=1,nob
         Cab(i)=0.0d0
        enddo
        do i=1,nob
         Cab(i)=abs(C(i))
        enddo
        if (nfun .eq. 1) then
         write(30,*)
         write(30,*) " Statistical analysis of the coefficients"
         write(30,2031) "  Sum(c):       ",sum(c(1:nob))
         write(30,2031) "  Aver(c):      ",sum(c(1:nob))/nob
         write(30,2031) "  Min(c):       ",minval(c(1:nob))
         write(30,2031) "  Max(c)        ",maxval(c(1:nob))
         write(30,2031) "  Sum(abs(c)):  ",sum(cab(1:nob))
         write(30,2031) "  Aver(abs(c)): ",sum(cab(1:nob))/nob
         write(30,2031) "  Min(abs(c)):  ",minval(cab(1:nob))
         write(30,2031) "  Max(abs(c)):  ",maxval(cab(1:nob))
        else if (nfun .eq. 2) then
         write(30,*)
         write(30,*) " Statistical analysis of the coefficients"
         write(30,*) ""
         write(30,*) "                  function A,         function B"
         write(30,2032) "  Sum(c):       ",sum(c(1:noba)),
     $             sum(c(noba+1:nob))
         write(30,2032) "  Aver(c):      ",sum(c(1:noba))/noba,
     $             sum(c(noba+1:nob))/nobb
         write(30,2032) "  Min(c):       ",minval(c(1:noba)),
     $             minval(c(noba+1:nob))
         write(30,2032) "  Max(c)        ",maxval(c(1:noba)),
     $             maxval(c(noba+1:nob))
         write(30,2032) "  Sum(abs(c)):  ",sum(cab(1:noba)),
     $             sum(cab(noba+1:nob))
         write(30,2032) "  Aver(abs(c)): ",sum(cab(1:noba))/noba,
     $             sum(cab(noba+1:nob))/nobb
         write(30,2032) "  Min(abs(c)):  ",minval(cab(1:noba)),
     $             minval(cab(noba+1:nob))
         write(30,2032) "  Max(abs(c)):  ",maxval(cab(1:noba)),
     $             maxval(cab(noba+1:nob))
        endif
      else if (ecor .eq. 1) then
        allocate(Cab(nob),Ccab(nobc))
        do i=1,nob
         Cab(i)=0.0d0
        enddo
        do i=1,nob
         Cab(i)=abs(C(i))
        enddo
        if (nfun .eq. 1) then
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vll"
         write(30,2031) "  Sum(c):       ",sum(c(1:nob))
         write(30,2031) "  Aver(c):      ",sum(c(1:nob))/nob
         write(30,2031) "  Min(c):       ",minval(c(1:nob))
         write(30,2031) "  Max(c)        ",maxval(c(1:nob))
         write(30,2031) "  Sum(abs(c)):  ",sum(cab(1:nob))
         write(30,2031) "  Aver(abs(c)): ",sum(cab(1:nob))/nob
         write(30,2031) "  Min(abs(c)):  ",minval(cab(1:nob))
         write(30,2031) "  Max(abs(c)):  ",maxval(cab(1:nob))
        else if (nfun .eq. 2) then
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vll"
         write(30,*) ""
         write(30,*) "                  function A,         function B"
         write(30,2032) "  Sum(c):       ",sum(c(1:noba)),
     $             sum(c(noba+1:nob))
         write(30,2032) "  Aver(c):      ",sum(c(1:noba))/noba,
     $             sum(c(noba+1:nob))/nobb
         write(30,2032) "  Min(c):       ",minval(c(1:noba)),
     $             minval(c(noba+1:nob))
         write(30,2032) "  Max(c)        ",maxval(c(1:noba)),
     $             maxval(c(noba+1:nob))
         write(30,2032) "  Sum(abs(c)):  ",sum(cab(1:noba)),
     $             sum(cab(noba+1:nob))
         write(30,2032) "  Aver(abs(c)): ",sum(cab(1:noba))/noba,
     $             sum(cab(noba+1:nob))/nobb
         write(30,2032) "  Min(abs(c)):  ",minval(cab(1:noba)),
     $             minval(cab(noba+1:nob))
         write(30,2032) "  Max(abs(c)):  ",maxval(cab(1:noba)),
     $             maxval(cab(noba+1:nob))
        endif

        do i=1,nobc
         Ccab(i)=0.0d0
        enddo
        do i=1,nobc
         Ccab(i)=abs(Ccor(i))
        enddo
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vhl"
         write(30,2031) "  Sum(c):       ",sum(ccor(1:nobc))
         write(30,2031) "  Aver(c):      ",sum(ccor(1:nobc))/nobc
         write(30,2031) "  Min(c):       ",minval(ccor(1:nobc))
         write(30,2031) "  Max(c)        ",maxval(ccor(1:nobc))
         write(30,2031) "  Sum(abs(c)):  ",sum(ccab(1:nobc))
         write(30,2031) "  Aver(abs(c)): ",sum(ccab(1:nobc))/nobc
         write(30,2031) "  Min(abs(c)):  ",minval(ccab(1:nobc))
         write(30,2031) "  Max(abs(c)):  ",maxval(ccab(1:nobc))
      else if (ecor .eq. 2) then
        allocate(Cab(nob))
        do i=1,nob
         Cab(i)=0.0d0
        enddo
        do i=1,nob
         Cab(i)=abs(C(i))
        enddo
        if (nfun .eq. 1) then
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vll"
         write(30,2031) "  Sum(c):       ",sum(c(1:nob))
         write(30,2031) "  Aver(c):      ",sum(c(1:nob))/nob
         write(30,2031) "  Min(c):       ",minval(c(1:nob))
         write(30,2031) "  Max(c)        ",maxval(c(1:nob))
         write(30,2031) "  Sum(abs(c)):  ",sum(cab(1:nob))
         write(30,2031) "  Aver(abs(c)): ",sum(cab(1:nob))/nob
         write(30,2031) "  Min(abs(c)):  ",minval(cab(1:nob))
         write(30,2031) "  Max(abs(c)):  ",maxval(cab(1:nob))
        else if (nfun .eq. 2) then
          do i=1,nob
           Cab(i)=abs(C(i))
          enddo
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vll"
         write(30,*) ""
         write(30,*) "                  function A,         function B"
         write(30,2032) "  Sum(c):       ",sum(c(1:noba)),
     $             sum(c(noba+1:nob))
         write(30,2032) "  Aver(c):      ",sum(c(1:noba))/noba,
     $             sum(c(noba+1:nob))/nobb
         write(30,2032) "  Min(c):       ",minval(c(1:noba)),
     $             minval(c(noba+1:nob))
         write(30,2032) "  Max(c)        ",maxval(c(1:noba)),
     $             maxval(c(noba+1:nob))
         write(30,2032) "  Sum(abs(c)):  ",sum(cab(1:noba)),
     $             sum(cab(noba+1:nob))
         write(30,2032) "  Aver(abs(c)): ",sum(cab(1:noba))/noba,
     $             sum(cab(noba+1:nob))/nobb
         write(30,2032) "  Min(abs(c)):  ",minval(cab(1:noba)),
     $             minval(cab(noba+1:nob))
         write(30,2032) "  Max(abs(c)):  ",maxval(cab(1:noba)),
     $             maxval(cab(noba+1:nob))
        endif
      else if (ecor .eq. 3) then
        allocate(Ccab(nobc))
        do i=1,nobc
         Ccab(i)=0.0d0
        enddo
        do i=1,nobc
         Ccab(i)=abs(Ccor(i))
        enddo
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vhl"
         write(30,2031) "  Sum(c):       ",sum(ccor(1:nobc))
         write(30,2031) "  Aver(c):      ",sum(ccor(1:nobc))/nobc
         write(30,2031) "  Min(c):       ",minval(ccor(1:nobc))
         write(30,2031) "  Max(c)        ",maxval(ccor(1:nobc))
         write(30,2031) "  Sum(abs(c)):  ",sum(ccab(1:nobc))
         write(30,2031) "  Aver(abs(c)): ",sum(ccab(1:nobc))/nobc
         write(30,2031) "  Min(abs(c)):  ",minval(ccab(1:nobc))
         write(30,2031) "  Max(abs(c)):  ",maxval(ccab(1:nobc))
      else if (ecor .eq. 4) then
        allocate(Cab(nob),Ccab(nobc))
        do i=1,nob
         Cab(i)=0.0d0
        enddo
        do i=1,nob
         Cab(i)=abs(C(i))
        enddo
        if (nfun .eq. 1) then
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vll"
         write(30,2031) "  Sum(c):       ",sum(c(1:nob))
         write(30,2031) "  Aver(c):      ",sum(c(1:nob))/nob
         write(30,2031) "  Min(c):       ",minval(c(1:nob))
         write(30,2031) "  Max(c)        ",maxval(c(1:nob))
         write(30,2031) "  Sum(abs(c)):  ",sum(cab(1:nob))
         write(30,2031) "  Aver(abs(c)): ",sum(cab(1:nob))/nob
         write(30,2031) "  Min(abs(c)):  ",minval(cab(1:nob))
         write(30,2031) "  Max(abs(c)):  ",maxval(cab(1:nob))
        else if (nfun .eq. 2) then
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vll"
         write(30,*) ""
         write(30,*) "                  function A,         function B"
         write(30,2032) "  Sum(c):       ",sum(c(1:noba)),
     $             sum(c(noba+1:nob))
         write(30,2032) "  Aver(c):      ",sum(c(1:noba))/noba,
     $             sum(c(noba+1:nob))/nobb
         write(30,2032) "  Min(c):       ",minval(c(1:noba)),
     $             minval(c(noba+1:nob))
         write(30,2032) "  Max(c)        ",maxval(c(1:noba)),
     $             maxval(c(noba+1:nob))
         write(30,2032) "  Sum(abs(c)):  ",sum(cab(1:noba)),
     $             sum(cab(noba+1:nob))
         write(30,2032) "  Aver(abs(c)): ",sum(cab(1:noba))/noba,
     $             sum(cab(noba+1:nob))/nobb
         write(30,2032) "  Min(abs(c)):  ",minval(cab(1:noba)),
     $             minval(cab(noba+1:nob))
         write(30,2032) "  Max(abs(c)):  ",maxval(cab(1:noba)),
     $             maxval(cab(noba+1:nob))
        endif
        do i=1,nobc
         Ccab(i)=0.0d0
        enddo
        do i=1,nobc
         Ccab(i)=abs(Ccor(i))
        enddo
         write(30,*)
         write(30,*) "Statistical analysis of the coefficients for Vhl"
         write(30,2031) "  Sum(c):       ",sum(ccor(1:nobc))
         write(30,2031) "  Aver(c):      ",sum(ccor(1:nobc))/nobc
         write(30,2031) "  Min(c):       ",minval(ccor(1:nobc))
         write(30,2031) "  Max(c)        ",maxval(ccor(1:nobc))
         write(30,2031) "  Sum(abs(c)):  ",sum(ccab(1:nobc))
         write(30,2031) "  Aver(abs(c)): ",sum(ccab(1:nobc))/nobc
         write(30,2031) "  Min(abs(c)):  ",minval(ccab(1:nobc))
         write(30,2031) "  Max(abs(c)):  ",maxval(ccab(1:nobc))
      endif
       endif

 2031 Format(a,E20.12)
 2032 Format(a,2E20.12)

      write(30,*)
      call getdat(iyr,imon,iday)
      call gettim(ihr,imin,isec,ihund)
      write(30,9040) iyr,imon,iday,ihr,imin,isec
 9040 Format(' Exit PIPFit at ',I4, '/',I2,'/',I2,','
     $       ,I2,':',I2,':',I2,'.')
      write(30,*) "Thank you for using PIPFit! Have a nice day!"
      write(30,*)

      end 

C selsor1 is a subroutine to give an ascending order for the bond
C lengths if itype=1 
      subroutine selsor1(rst)
      integer k, l, locsm
      double precision,dimension(:):: rst(6)
      double precision sml

      sml=0.0d0

      do k=1,5
      sml = rst(k)      
      locsm = k
      do l=k+1,6
        if (rst(l) .LT. sml) then
        sml = rst(l)
        locsm=l
        endif
      enddo
      rst(locsm) = rst(k)
      rst(k) = sml
      enddo

      end subroutine

C selsor1 is a subroutine to give an ascending order for the bond
C lengths if itype=3
      subroutine selsor3(rst)
      integer k, l, locsm
      double precision,dimension(:):: rst(6)
      double precision sml

      sml=0.0d0

      do k=2,4
      sml = rst(k)
      locsm = k
      do l=k+1,5
        if (rst(l) .LT. sml) then
        sml = rst(l)
        locsm=l
        endif
      enddo
      rst(locsm) = rst(k)
      rst(k) = sml
      enddo

      end subroutine

