      subroutine readinp(iordera,iorderb,iorderc,iorderd,rcond,pairw,
     $                   ecor,ired,nred)
***********************************************************************
*  Program to calculate the XYZ coordinate by reading input
*  IOrder       Degree of polynomial basis functions
*  IOrderA      Degree of polynomial basis functions for func A
*  IOrderB      Degree of polynomial basis functions for func B
*  IOrderC      Degree of polynomial basis functions for func C
*  IOrderD      Degree of polynomial basis functions for func D, inact.
*
*  NFun = 1    Using only one IOrder function
*       = 2    Using two IOrder functions (A and B)  -VZ
*
*   IMGA = 1     Morse function 
*          2     Gauss function
*          3     MEG function  exp(-x/a-x^2/b) 
*          4     sech based function  1/cosh(x/a)
*   IMGB =       same as IMGA for the second funtion if NFun=2
*   IMGC =       same as IMGA for the Vhl funtion 
*   IMGD =       same as IMGA, currently inactive
*
*  ITyp = 1     A4 system       (IOrder = 5-12)
*       = 2     AB3 system      (IOrder = 5-10)
*       = 3     A2B2 system     (IOrder = 5-12)
*       = 4     A2BC system     (IOrder = 5-10)
*       = 5     ABCD system     (IOrder = 5-9)
*
*  ICop = 0     No constraints
*       = 1     Force C0 = V0
*               (Energy of four infinite seperated atoms)
*
*  IMethd = 0   Bowman-Braams approach
*         = 1   Remove the unconnected terms
*         = 2   Remove the 2-body and unconnected terms
*               (ICop need to be 0 for IMethd = 2)
*
*  IWop = 0     No wighting funtions used
*       = 1     w=1.0 for V<Ecr; w=(Ecr/V)^epow
*       = 2     w=1.0 for V<Ecr; w=(Ecr/V)^epow and then w is multipled by average of d
*               d = sqrt(sum((r-r')/((r+r')))^2/6)
*               where r distance of a given geometry and r' belongs to all 
*               of the other geometries in the data set
*       = 3     w=1.0 for V<Ecr; w=(V/Ecr)^epow
*       = 4     w=1.0 for V<Ecr; w=(V/Ecr)^epow and then w is multipled by average of d
*               d = sqrt(sum((r-r')/((r+r')))^2/6)
*               where r distance of a given geometry and r' belongs to all 
*  Ecr:         Critical energy value for option in wighting funtion IWop ((Ecr/V)^epow) 
*  Ecrsh:       In equation (Ecr/V)^epow, Ecrsh can shift the value of V
*               if, for instance, three atom systems are calculated with
*               the equation set of four atoms  
*  Epow:        Power value for option in wighting funtion IWop ((Ecr/V)^epow)
*
*
*  ICoord = 0   Internal (Jacob) coordinate
*         = 1   Cartesian coordinate
*         = 2   Read the internal coordinates (r)
*
*  IFmt   = 0   The ab initio set does not include data index
*         = 1   The ab initio set includes data index
*
*  Iout = 0     Write .vf file with ab initio and fitted energies
*               for the geometries in .abn file 
*       = 1     Write .vfe file too with ab initio, fitted, and diatomic  energies 
*               without geometries
*       = 2     Write .vfe2 file too for the terms of the multinomial
*               function (USE ONLY REDUCED GEOMETRY SET DUE TO THE
*               LARGE FILE SIZE!!!)
*
*  Isan = 1     Do a statistical analyzis for the coeffs - VZ 043014
*       = 0     Skip the analyzis
*
*  miel:        Min. Vabit energy border for the list in errdist.out -VZ
*  mael:        Max. Vabit energy border for the list in errdist.out -VZ
*  ael:         Min. absoult energy limit for the listed errors in errdist.out -VZ
*
* erange(4):    Energy borders to get five energy bins for statistical
*               analyzis   -VZ
*
* grange(4):    Geometry borders to get five geometry bins for statistical
*               analyzis, sum(r(i))/6  -VZ
*
* pairw = 1     Generalized Morse potentials are used for pair-wise
*               interactions (it was used for N4 system)
*       = 2     MEG potentials are used for NO and O2 pair-wises, and
*               Generalized Morse potential is used for N2 in N2O2 and
*               N2O systems
*       = 3     Even tempered Gaussian short-range terms were used for O2
*               in O4 and O3 systems, where the short-range term is described
*               by D3 dispersion 
*
* Ecor  = 0     Vtotal is fitted
*       = 1     Vll and Vhl are fitted separately
*               but read from the same source file       
*       = 2     Vll is fitted alone
*       = 3     Vhl is fitted alone
*       = 4     Vll and Ehl are fitted separately
*               and read from different sources
*
***********************************************************************
      implicit double precision (a-h,o-z)

      common /inout/ icoord,ifmt,iout,isan
      common /optn/ iatyp,icop,imethd,iwop,ityp,nfun,imga,imgb,imgc,imgd
      common /energy/ ecr,esh,epow,elimit,erange(4),grange(4),v0
      common /msprmt/  arng(6),brng(6),crng(6),drng(6)
      common /msprmt3/ erng(6),frng(6),grng(6),hrng(6)
      common /msprmt2/ ra(6),rb(6),rc(6),rd(6),re(6),rf(6),rg(6),rh(6)
      common /errlist/ smiel, smael, sael

      character (len=50) uinp,abnu,outu,erru,cffu,vfu,vfeu,vfe2u,
     $                   abnucas,abnucor,vfcas,vfcor,cffucas,cffucor
      common /filnam/ uinp,abnu,outu,erru,cffu,vfu,vfeu,vfe2u,
     $                   abnucas,abnucor,vfcas,vfcor,cffucas,cffucor

      integer i,j,io,n
      integer indx,iout
      integer iordera,iorderb,iorderc,iorderd
      integer nopm,nop
      integer pairw,ecor,ired,nred
      double precision rcond
      logical fe1,fe2,fe3,fe4
C      character (len=50) uinp,abnu
      character (len=100) record1,records

C Default parameter values
      ityp      = 1
      iatyp     = 56
      nfun      = 1
      imga      = 1
      imgb      = 1
      imgc      = 1
      imgd      = 1
      iordera   = 5
      iorderb   = 0
      iorderc   = 0
      iorderd   = 0
      imethd    = 2
      icop      = 0
      iwop      = 0
      icoord    = 0
      ifmt      = 0
      iout      = 0
      isan      = 0
      ired      = 0
      nred      = 5
      elimit    = 2000.0
      erange(1)  = 100.0
      erange(2)  = 200.0
      erange(3)  = 500.0
      erange(4)  = 1000.0
      grange(1)  = 1.1
      grange(2)  = 1.6
      grange(3)  = 2.3
      grange(4)  = 3.2
      sael      = 30.0
      smiel     = -50.0
      smael     = 500.0
      rcond     = 1.0d-15
      ra(1)=1.0d0
      ra(2)=1.0d0
      ra(3)=1.0d0
      ra(4)=1.0d0
      ra(5)=1.0d0
      ra(6)=1.0d0
      rb(1)=1.0d0
      rb(2)=1.0d0
      rb(3)=1.0d0
      rb(4)=1.0d0
      rb(5)=1.0d0
      rb(6)=1.0d0
      rc(1)=1.0d0
      rc(2)=1.0d0
      rc(3)=1.0d0
      rc(4)=1.0d0
      rc(5)=1.0d0
      rc(6)=1.0d0
      rd(1)=1.0d0
      rd(2)=1.0d0
      rd(3)=1.0d0
      rd(4)=1.0d0
      rd(5)=1.0d0
      rd(6)=1.0d0
      re(1)=1.0d0
      re(2)=1.0d0
      re(3)=1.0d0
      re(4)=1.0d0
      re(5)=1.0d0
      re(6)=1.0d0
      rf(1)=1.0d0
      rf(2)=1.0d0
      rf(3)=1.0d0
      rf(4)=1.0d0
      rf(5)=1.0d0
      rf(6)=1.0d0
      rg(1)=1.0d0
      rg(2)=1.0d0
      rg(3)=1.0d0
      rg(4)=1.0d0
      rg(5)=1.0d0
      rg(6)=1.0d0
      rh(1)=1.0d0
      rh(2)=1.0d0
      rh(3)=1.0d0
      rh(4)=1.0d0
      rh(5)=1.0d0
      rh(6)=1.0d0
      arng(1)=1.0d0
      arng(2)=1.0d0
      arng(3)=1.0d0
      arng(4)=1.0d0
      arng(5)=1.0d0
      arng(6)=1.0d0
      brng(1)=1.0d0
      brng(2)=1.0d0
      brng(3)=1.0d0
      brng(4)=1.0d0
      brng(5)=1.0d0
      brng(6)=1.0d0
      crng(1)=1.0d0
      crng(2)=1.0d0
      crng(3)=1.0d0
      crng(4)=1.0d0
      crng(5)=1.0d0
      crng(6)=1.0d0
      drng(1)=1.0d0
      drng(2)=1.0d0
      drng(3)=1.0d0
      drng(4)=1.0d0
      drng(5)=1.0d0
      drng(6)=1.0d0
      erng(1)=1.0d0
      erng(2)=1.0d0
      erng(3)=1.0d0
      erng(4)=1.0d0
      erng(5)=1.0d0
      erng(6)=1.0d0
      frng(1)=1.0d0
      frng(2)=1.0d0
      frng(3)=1.0d0
      frng(4)=1.0d0
      frng(5)=1.0d0
      frng(6)=1.0d0
      grng(1)=1.0d0
      grng(2)=1.0d0
      grng(3)=1.0d0
      grng(4)=1.0d0
      grng(5)=1.0d0
      grng(6)=1.0d0
      hrng(1)=1.0d0
      hrng(2)=1.0d0
      hrng(3)=1.0d0
      hrng(4)=1.0d0
      hrng(5)=1.0d0
      hrng(6)=1.0d0
      pairw = 1
      ecor = 0
      ecr = 0.0d0
      esh = 0.0d0
      epow = 2.0d0

C First read parameters and constraints
C Determine the type of system
C new format and entries were introduced  - VZ 043014

      open(unit=10,file=trim(adjustl(uinp)),status='old')
C File 12, store the input without comment
      open(12,status='scratch')
C      open(12,status='new')
C File 13, file used to read in variables
      open(13,status='scratch')
C      open(13,status='new')
      
C Read input and write into temporary file 12 and convert captical
C characters to lower case ones
      do while (.true.)
        read(10,'(a)',end=100) records
        if (records(1:1) .ne. '#') then
          call change_case(records,record1,0)
          write(12,'(a)') record1
        endif
      enddo

  100 continue 

      write(12,'(a)') "thend0"
      rewind(12)
      close(10)

C Read input variables
C The formats of inupt:
C 1. $system var1=1 var2=2 $end
C 2. $system
C      var1=1
C      var2=2
C     $end

      do while(.true.)
        read(12,'(a)',end=200) records
          if (index(records,'$molecule') .gt. 0) then
            endreccon=0
            do while(endreccon .ne. 9999)
              ifkeyexist=index(records,'ityp')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ityp')+1:)
                rewind(13)
                read(13,*) ityp
                rewind(13)
              endif
              ifkeyexist=index(records,'iatyp')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('iatyp')+1:)
                rewind(13)
                read(13,*) iatyp
                rewind(13)
              endif
              ifkeyexist=index(records,'ra1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ra1')+1:)
                rewind(13)
                read(13,*) ra(1)
                rewind(13) 
              endif
              ifkeyexist=index(records,'ra2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ra2')+1:)
                rewind(13)
                read(13,*) ra(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'ra3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ra3')+1:)
                rewind(13)
                read(13,*) ra(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'ra4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ra4')+1:)
                rewind(13)
                read(13,*) ra(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'ra5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ra5')+1:)
                rewind(13)
                read(13,*) ra(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'ra6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ra6')+1:)
                rewind(13)
                read(13,*) ra(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'rb1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rb1')+1:)
                rewind(13)
                read(13,*) rb(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'rb2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rb2')+1:)
                rewind(13)
                read(13,*) rb(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'rb3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rb3')+1:)
                rewind(13)
                read(13,*) rb(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'rb4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rb4')+1:)
                rewind(13)
                read(13,*) rb(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'rb5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rb5')+1:)
                rewind(13)
                read(13,*) rb(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'rb6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rb6')+1:)
                rewind(13)
                read(13,*) rb(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'rc1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rc1')+1:)
                rewind(13)
                read(13,*) rc(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'rc2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rc2')+1:)
                rewind(13)
                read(13,*) rc(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'rc3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rc3')+1:)
                rewind(13)
                read(13,*) rc(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'rc4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rc4')+1:)
                rewind(13)
                read(13,*) rc(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'rc5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rc5')+1:)
                rewind(13)
                read(13,*) rc(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'rc6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rc6')+1:)
                rewind(13)
                read(13,*) rc(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'rd1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rd1')+1:)
                rewind(13)
                read(13,*) rd(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'rd2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rd2')+1:)
                rewind(13)
                read(13,*) rd(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'rd3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rd3')+1:)
                rewind(13)
                read(13,*) rd(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'rd4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rd4')+1:)
                rewind(13)
                read(13,*) rd(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'rd5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rd5')+1:)
                rewind(13)
                read(13,*) rd(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'rd6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rd6')+1:)
                rewind(13)
                read(13,*) rd(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'re1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('re1')+1:)
                rewind(13)
                read(13,*) re(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'re2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('re2')+1:)
                rewind(13)
                read(13,*) re(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'re3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('re3')+1:)
                rewind(13)
                read(13,*) re(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'re4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('re4')+1:)
                rewind(13)
                read(13,*) re(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'re5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('re5')+1:)
                rewind(13)
                read(13,*) re(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'re6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('re6')+1:)
                rewind(13)
                read(13,*) re(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'rf1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rf1')+1:)
                rewind(13)
                read(13,*) rf(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'rf2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rf2')+1:)
                rewind(13)
                read(13,*) rf(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'rf3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rf3')+1:)
                rewind(13)
                read(13,*) rf(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'rf4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rf4')+1:)
                rewind(13)
                read(13,*) rf(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'rf5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rf5')+1:)
                rewind(13)
                read(13,*) rf(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'rf6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rf6')+1:)
                rewind(13)
                read(13,*) rf(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'rg1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rg1')+1:)
                rewind(13)
                read(13,*) rg(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'rg2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rg2')+1:)
                rewind(13)
                read(13,*) rg(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'rg3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rg3')+1:)
                rewind(13)
                read(13,*) rg(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'rg4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rg4')+1:)
                rewind(13)
                read(13,*) rg(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'rg5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rg5')+1:)
                rewind(13)
                read(13,*) rg(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'rg6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rg6')+1:)
                rewind(13)
                read(13,*) rg(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'rh1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rh1')+1:)
                rewind(13)
                read(13,*) rh(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'rh2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rh2')+1:)
                rewind(13)
                read(13,*) rh(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'rh3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rh3')+1:)
                rewind(13)
                read(13,*) rh(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'rh4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rh4')+1:)
                rewind(13)
                read(13,*) rh(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'rh5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rh5')+1:)
                rewind(13)
                read(13,*) rh(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'rh6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rh6')+1:)
                rewind(13)
                read(13,*) rh(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'arng1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('arng1')+1:)
                rewind(13)
                read(13,*) arng(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'arng2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('arng2')+1:)
                rewind(13)
                read(13,*) arng(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'arng3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('arng3')+1:)
                rewind(13)
                read(13,*) arng(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'arng4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('arng4')+1:)
                rewind(13)
                read(13,*) arng(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'arng5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('arng5')+1:)
                rewind(13)
                read(13,*) arng(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'arng6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('arng6')+1:)
                rewind(13)
                read(13,*) arng(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'brng1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('brng1')+1:)
                rewind(13)
                read(13,*) brng(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'brng2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('brng2')+1:)
                rewind(13)
                read(13,*) brng(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'brng3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('brng3')+1:)
                rewind(13)
                read(13,*) brng(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'brng4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('brng4')+1:)
                rewind(13)
                read(13,*) brng(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'brng5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('brng5')+1:)
                rewind(13)
                read(13,*) brng(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'brng6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('brng6')+1:)
                rewind(13)
                read(13,*) brng(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'crng1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('crng1')+1:)
                rewind(13)
                read(13,*) crng(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'crng2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('crng2')+1:)
                rewind(13)
                read(13,*) crng(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'crng3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('crng3')+1:)
                rewind(13)
                read(13,*) crng(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'crng4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('crng4')+1:)
                rewind(13)
                read(13,*) crng(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'crng5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('crng5')+1:)
                rewind(13)
                read(13,*) crng(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'crng6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('crng6')+1:)
                rewind(13)
                read(13,*) crng(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'drng1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('drng1')+1:)
                rewind(13)
                read(13,*) drng(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'drng2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('drng2')+1:)
                rewind(13)
                read(13,*) drng(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'drng3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('drng3')+1:)
                rewind(13)
                read(13,*) drng(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'drng4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('drng4')+1:)
                rewind(13)
                read(13,*) drng(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'drng5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('drng5')+1:)
                rewind(13)
                read(13,*) drng(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'drng6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('drng6')+1:)
                rewind(13)
                read(13,*) drng(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'erng1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erng1')+1:)
                rewind(13)
                read(13,*) erng(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'erng2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erng2')+1:)
                rewind(13)
                read(13,*) erng(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'erng3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erng3')+1:)
                rewind(13)
                read(13,*) erng(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'erng4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erng4')+1:)
                rewind(13)
                read(13,*) erng(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'erng5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erng5')+1:)
                rewind(13)
                read(13,*) erng(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'erng6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erng6')+1:)
                rewind(13)
                read(13,*) erng(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'frng1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('frng1')+1:)
                rewind(13)
                read(13,*) frng(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'frng2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('frng2')+1:)
                rewind(13)
                read(13,*) frng(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'frng3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('frng3')+1:)
                rewind(13)
                read(13,*) frng(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'frng4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('frng4')+1:)
                rewind(13)
                read(13,*) frng(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'frng5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('frng5')+1:)
                rewind(13)
                read(13,*) frng(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'frng6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('frng6')+1:)
                rewind(13)
                read(13,*) frng(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'grng1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grng1')+1:)
                rewind(13)
                read(13,*) grng(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'grng2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grng2')+1:)
                rewind(13)
                read(13,*) grng(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'grng3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grng3')+1:)
                rewind(13)
                read(13,*) grng(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'grng4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grng4')+1:)
                rewind(13)
                read(13,*) grng(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'grng5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grng5')+1:)
                rewind(13)
                read(13,*) grng(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'grng6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grng6')+1:)
                rewind(13)
                read(13,*) grng(6)
                rewind(13)
              endif
              ifkeyexist=index(records,'hrng1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('hrng1')+1:)
                rewind(13)
                read(13,*) hrng(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'hrng2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('hrng2')+1:)
                rewind(13)
                read(13,*) hrng(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'hrng3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('hrng3')+1:)
                rewind(13)
                read(13,*) hrng(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'hrng4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('hrng4')+1:)
                rewind(13)
                read(13,*) hrng(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'hrng5')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('hrng5')+1:)
                rewind(13)
                read(13,*) hrng(5)
                rewind(13)
              endif
              ifkeyexist=index(records,'hrng6')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('hrng6')+1:)
                rewind(13)
                read(13,*) hrng(6)
                rewind(13)
              endif
C Meet the end of this section
              if (index(records,'$end') .gt. 0) then
                endreccon=9999
              else
                read(12,'(a)',end=200) records
              endif
            enddo        
          endif
C Read input for fitting functions
          if (index(records,'$fitfunct') .gt. 0) then
            endreccon=0
            do while(endreccon .ne. 9999)
              ifkeyexist=index(records,'nfun')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('nfun')+1:)
                rewind(13)
                read(13,*) nfun
                rewind(13)
              endif
              ifkeyexist=index(records,'imga')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('imga')+1:)
                rewind(13)
                read(13,*) imga
                rewind(13)
              endif
              ifkeyexist=index(records,'imgb')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('imgb')+1:)
                rewind(13)
                read(13,*) imgb
                rewind(13)
              endif
              ifkeyexist=index(records,'imgc')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('imgc')+1:)
                rewind(13)
                read(13,*) imgc
                rewind(13)
              endif
              ifkeyexist=index(records,'imgd')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('imgd')+1:)
                rewind(13)
                read(13,*) imgd
                rewind(13)
              endif
C              ifkeyexist=index(records,'iorder')
C              if (ifkeyexist .gt. 0) then
C                write(13,*) records(ifkeyexist+len('iorder')+1:)
C                rewind(13)
C                read(13,*) iorder
C                rewind(13)
C              endif
              ifkeyexist=index(records,'iordera')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('iordera')+1:)
                rewind(13)
                read(13,*) iordera
                rewind(13)
              endif
              ifkeyexist=index(records,'iorderb')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('iorderb')+1:)
                rewind(13)
                read(13,*) iorderb
                rewind(13)
              endif
              ifkeyexist=index(records,'iorderc')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('iorderc')+1:)
                rewind(13)
                read(13,*) iorderc
                rewind(13)
              endif
              ifkeyexist=index(records,'iorderd')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('iorderd')+1:)
                rewind(13)
                read(13,*) iorderd
                rewind(13)
              endif
              ifkeyexist=index(records,'imethod')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('imethod')+1:)
                rewind(13)
                read(13,*) imethd
                rewind(13)
              endif
              ifkeyexist=index(records,'pairw')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('pairw')+1:)
                rewind(13)
                read(13,*) pairw 
                rewind(13)
              endif
              ifkeyexist=index(records,'ecor')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ecor')+1:)
                rewind(13)
                read(13,*) ecor 
                rewind(13)
              endif
              ifkeyexist=index(records,'icop')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('icop')+1:)
                rewind(13)
                read(13,*) icop
                rewind(13)
              endif
              ifkeyexist=index(records,'v0')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('v0')+1:)
                rewind(13)
                read(13,*) v0
                rewind(13)
              endif
              ifkeyexist=index(records,'rcond')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('rcond')+1:)
                rewind(13)
                read(13,*) rcond
                rewind(13)
              endif
              if (index(records,'$end') .gt. 0) then
                endreccon=9999
              else
                read(12,'(a)',end=200) records
              endif
            enddo
          endif
C Read input for energy ralated parameters
          if (index(records,'$energy') .gt. 0) then
            endreccon=0
            do while(endreccon .ne. 9999)
              ifkeyexist=index(records,'iwop')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('iwop')+1:)
                rewind(13)
                read(13,*) iwop
                rewind(13)
              endif
              ifkeyexist=index(records,'ecr')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ecr')+1:)
                rewind(13)
                read(13,*) ecr
                rewind(13)
              endif
              ifkeyexist=index(records,'esh')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('esh')+1:)
                rewind(13)
                read(13,*) esh
                rewind(13)
              endif
              ifkeyexist=index(records,'epow')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('epow')+1:)
                rewind(13)
                read(13,*) epow
                rewind(13)
              endif
              ifkeyexist=index(records,'elimit')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('elimit')+1:)
                rewind(13)
                read(13,*) elimit
                rewind(13)
              endif
              ifkeyexist=index(records,'erange1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erange1')+1:)
                rewind(13)
                read(13,*) erange(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'erange2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erange2')+1:)
                rewind(13)
                read(13,*) erange(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'erange3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erange3')+1:)
                rewind(13)
                read(13,*) erange(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'erange4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('erange4')+1:)
                rewind(13)
                read(13,*) erange(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'grange1')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grange1')+1:)
                rewind(13)
                read(13,*) grange(1)
                rewind(13)
              endif
              ifkeyexist=index(records,'grange2')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grange2')+1:)
                rewind(13)
                read(13,*) grange(2)
                rewind(13)
              endif
              ifkeyexist=index(records,'grange3')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grange3')+1:)
                rewind(13)
                read(13,*) grange(3)
                rewind(13)
              endif
              ifkeyexist=index(records,'grange4')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('grange4')+1:)
                rewind(13)
                read(13,*) grange(4)
                rewind(13)
              endif
              ifkeyexist=index(records,'smiel')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('smiel')+1:)
                rewind(13)
                read(13,*) smiel
                rewind(13)
              endif
              ifkeyexist=index(records,'smael')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('smael')+1:)
                rewind(13)
                read(13,*) smael
                rewind(13)
              endif
              ifkeyexist=index(records,'sael')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('sael')+1:)
                rewind(13)
                read(13,*) sael
                rewind(13)
              endif
              if (index(records,'$end') .gt. 0) then
                endreccon=9999
              else
                read(12,'(a)',end=200) records
              endif
            enddo
          endif
C Read input for input & output ralated parameters
          if (index(records,'$inout') .gt. 0) then
            endreccon=0
            do while(endreccon .ne. 9999)
              ifkeyexist=index(records,'ifmt')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ifmt')+1:)
                rewind(13)
                read(13,*) ifmt
                rewind(13)
              endif
              ifkeyexist=index(records,'icoord')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('icoord')+1:)
                rewind(13)
                read(13,*) icoord
                rewind(13)
              endif
              ifkeyexist=index(records,'iout')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('iout')+1:)
                rewind(13)
                read(13,*) iout
                rewind(13)
              endif
              ifkeyexist=index(records,'isan')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('isan')+1:)
                rewind(13)
                read(13,*) isan
                rewind(13)
              endif
              ifkeyexist=index(records,'ired')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('ired')+1:)
                rewind(13)
                read(13,*) ired
                rewind(13)
              endif
              ifkeyexist=index(records,'nred')
              if (ifkeyexist .gt. 0) then
                write(13,*) records(ifkeyexist+len('nred')+1:)
                rewind(13)
                read(13,*) nred
                rewind(13)
              endif
              if (index(records,'$end') .gt. 0) then
                endreccon=9999
              else
                read(12,'(a)',end=200) records
              endif
            enddo
          endif
C stop reading when the end of file 12 is reached
        if (index(records,'thend0') .gt. 0) go to 200
      enddo

  200 continue                

      close(12)
      close(13)

C Check for consistence
       if (nfun.gt.2) then
         write(*,*) "Incorrect NFun! Only 1 and 2 allowed!"
         stop
       endif

       if (img.ge.5) then
         write(*,*) "Incorrect IMG! Only 1, 2, 3, or 4 allowed!"
         stop
       endif

       call itypcopy(ra,arng,ityp)
       call itypcopy(rb,brng,ityp)
       call itypcopy(rc,crng,ityp)
       call itypcopy(rd,drng,ityp)
       call itypcopy(re,erng,ityp)
       call itypcopy(rf,frng,ityp)
       call itypcopy(rg,grng,ityp)
       call itypcopy(rh,hrng,ityp)

      if (imethd .gt. 3) then
        write(*,'(a)') ' Wrong Imethd, it must be 0, 1, 2, or 3'
        stop
      endif
      if (imethd .eq. 2) then
        if (iatyp .eq. 56) then
          write(30,'(a)') ' N4 system is considered.'
        else if (iatyp .eq. 60) then
          write(30,'(a)') ' N2O2 system is considered.'
        else if (iatyp .eq. 64) then
          write(30,'(a)') ' O4 system is considered.'
        else
          write(30,'(a)') 'Only 56, 60, or 64 is allowed for IATyp!'
        endif
      endif 
      if (pariw .gt. 3) then
       write(*,'(a)') ' Wrong pairw, it must be 1, 2, or 3'
        stop
      endif 
      if (icop .gt. 1) then
        write(*,*) "ICop should be chosen to be 0 or 1!"
        stop
      endif
      if (iwop .gt. 4) then
        write(*,*) "IWop should be chosen to be 0, 1, 2, 3 and 4!"
        stop
      endif
C Check iout and isan
      if (iout .gt. 2) then
        write(*,'(a)') ' Wrong Iout, it must be 0, 1, or 2'
        stop
      endif
      if (isan .gt. 1) then
        write(*,'(a)') ' Wrong IEAN, it must be 0 or 1'
        stop
      endif
C Check ifmt and icoord 
      if (ifmt .gt. 1) then
        write(*,'(a)') ' Wrong Ifmt, it must be 0 or 1'
        stop
      endif
      if (icoord .gt. 2) then
        write(*,'(a)') ' Wrong ICoord, it must be 0, 1, or 2'
        stop
      endif

      end

       subroutine itypcopy(rx,xrng,ty)
       double precision rx(6), xrng(6)
       integer ty

       if (ty .eq. 1) then
         do i=2,6
           xrng(i)=xrng(1)
           rx(i)=rx(1)
         enddo
       else if (ty .eq. 2) then
         xrng(2)=xrng(1)
         xrng(3)=xrng(1)
         xrng(5)=xrng(4)
         xrng(6)=xrng(4)
         rx(2)=rx(1)
         rx(3)=rx(1)
         rx(5)=rx(4)
         rx(6)=rx(4)
       else if (ty .eq. 3) then
         xrng(3)=xrng(2)
         xrng(4)=xrng(2)
         xrng(5)=xrng(2)
         rx(3)=rx(2)
         rx(4)=rx(2)
         rx(5)=rx(2)
       else if (ty .eq. 4) then
         xrng(4)=xrng(2)
         xrng(5)=xrng(3)
         rx(4)=rx(2)
         rx(5)=rx(3)
C Do nonthing for ityp = 5
       else if (ityp .ge. 5) then
         write(*,*) "Incorrect ITyp! Only 1, 2, 3, and 4 allowed!"
         stop
       endif

       end subroutine itypcopy

